@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -13,6 +13,7 @@
 #include "PWSprefs.h"
 
 #include <algorithm>
+#include <iterator>
 
 // ------------------------------------------------
 // Base class: Command
@@ -44,42 +45,58 @@
 using pws_os::CUUID;
 
 Command::Command(CommandInterface *pcomInt)
-:  m_pcomInt(pcomInt), m_bNotifyGUI(true), m_RC(0), m_CommandDBChange(NONE)
+:  m_pcomInt(pcomInt), m_bNotifyGUI(true), m_bInMultiCommand(false),
+   m_RC(0), m_CommandChangeType(NONE), m_CommandDBChange(NONE)
 {
-  m_PreCommand.Clear();
-  m_PostCommand.Clear();
-  m_Command.Clear();
 }
 
 Command::~Command()
 {
 }
 
-// The following should only be called from PWScore Execute/Undo/Redo
-void Command::SaveChangedState(StateType st, st_DBChangeStatus &stDBCS)
+void Command::SaveDBInformation()
 {
-  switch (st) {
-    case COMMANDACTION:
-      // Set what the command changes
-      m_Command = stDBCS;
-      break;
-    case PREEXECUTE:
-      // Save current pre-command execute state
-      m_PreCommand = stDBCS;
-      break;
-    case POSTEXECUTE:
-      // Here we generate new state from pre-execute state and command action states
-      // Effectively post-execute state = pre-execute state + command action state
-      // and, unlike the other calls, return the result
-      stDBCS = m_PostCommand = (m_PreCommand + m_Command);
-      break;
+  // Currently only modified nodes are dealt with - could add any other DB information
+  // at a later date if required
+  // right predicate's only relevant for nested MultiCommands
+  if (!InMultiCommand() || (dynamic_cast<MultiCommands *>(this) != NULL)) {
+    // Only do this if executed outside a MultiCommand or it is the Multicommand itself
+    // We could change an entry and so here is where we save DB information
+    // just in case.  Currently only modified nodes.
+    m_vSavedModifiedNodes = m_pcomInt->GetModifiedNodes();
   }
 }
 
-// The following should only be called from PWScore Execute/Undo/Redo
-void Command::RestoreChangedState(st_DBChangeStatus &stDBCS)
+void Command::RestoreDBInformation()
 {
-  stDBCS = m_PreCommand;
+  // Only do this if executed outside a MultiCommand or it is the Multicommand itself
+  // Currently only modified nodes are dealt with - could add any other DB information
+  // at a later date if required
+  // right predicate's only relevant for nested MultiCommands
+  if (!InMultiCommand() || (dynamic_cast<MultiCommands *>(this) != NULL)) {
+    // Get current modified nodes vector
+    std::vector<StringX> vModifiedNodes = m_pcomInt->GetModifiedNodes();
+
+    // Only do this if executed outside a MultiCommand
+    // We could change an entry and so here is where we save DB information
+    // just in case.  Currently only modified nodes.
+    m_pcomInt->SetModifiedNodes(m_vSavedModifiedNodes);
+
+    // We now have to refresh those modified groups now no longer modified
+    std::sort(vModifiedNodes.begin(), vModifiedNodes.end());
+    std::sort(m_vSavedModifiedNodes.begin(), m_vSavedModifiedNodes.end());
+
+    // Remove those modified nodes that were modified before we executed this command
+    std::vector<StringX> vChangedNodes;
+    std::set_difference(
+      vModifiedNodes.begin(), vModifiedNodes.end(),
+      m_vSavedModifiedNodes.begin(), m_vSavedModifiedNodes.end(),
+      std::inserter(vChangedNodes, vChangedNodes.begin())
+    );
+
+    // Tell GUI to refresh this group
+    m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_REFRESH_GROUPS, vChangedNodes);
+  }
 }
 
 // ------------------------------------------------
@@ -89,26 +106,49 @@ void Command::RestoreChangedState(st_DBChangeStatus &stDBCS)
 MultiCommands::MultiCommands(CommandInterface *pcomInt)
   : Command(pcomInt)
 {
-  m_CommandDBChange = MULTICOMMAND;
 }
 
 MultiCommands::~MultiCommands()
 {
-  std::vector<Command *>::iterator cmd_Iter;
+  for_each(m_vpcmds.begin(), m_vpcmds.end(),
+           [] (Command *pcmd) {delete pcmd;});
+}
 
-  for (cmd_Iter = m_vpcmds.begin(); cmd_Iter != m_vpcmds.end(); cmd_Iter++) {
-    delete (*cmd_Iter);
-  }
+Command *MultiCommands::FindCommand(const std::type_info &ti)
+{
+  // Initial implementation - search for first command of a specific class
+  // in a MultiCommand.  Could be enhanced to return a vector of commands
+  // if all commands of a specific class are required.
+
+  auto retval = find_if(m_vpcmds.begin(), m_vpcmds.end(),
+                        [&ti] (Command *pcmd) {
+                          return typeid(*pcmd) == ti;
+                        }
+                        );
+
+  return (retval != m_vpcmds.end()) ? *retval : nullptr;
 }
 
 int MultiCommands::Execute()
 {
   std::vector<Command *>::iterator cmd_Iter;
 
+  for (cmd_Iter = m_vpcmds.begin(); cmd_Iter != m_vpcmds.end(); cmd_Iter++) {
+    if (*cmd_Iter != NULL && (*cmd_Iter)->IsEntryChangeType()) {
+      // We could change an entry and so here is where we save DB information
+      // just in case.  Currently only modified nodes.
+      SaveDBInformation();
+      break;
+    }
+  }
+
   for (cmd_Iter = m_vpcmds.begin(); cmd_Iter != m_vpcmds.end(); cmd_Iter++) {
     int rc(-1);
     if (*cmd_Iter != NULL) {
       rc = (*cmd_Iter)->Execute();
+
+      if ((*cmd_Iter)->WasDBChanged())
+        m_CommandDBChange = CommandDBChange::MULTICOMMAND;
     }
     m_vRCs.push_back(rc);
   }
@@ -124,44 +164,31 @@ void MultiCommands::Undo()
     if (*cmd_rIter != NULL)
       (*cmd_rIter)->Undo();
   }
+
+  for (cmd_rIter = m_vpcmds.rbegin(); cmd_rIter != m_vpcmds.rend(); cmd_rIter++) {
+    if (*cmd_rIter != NULL && (*cmd_rIter)->IsEntryChangeType()) {
+      // We could change an entry and so here is where we save DB information
+      // just in case.  Currently only modified nodes.
+      RestoreDBInformation();
+      break;
+    }
+  }
 }
 
 void MultiCommands::Add(Command *pcmd)
 {
   ASSERT(pcmd != NULL);
+  pcmd->SetInMultiCommand();
   m_vpcmds.push_back(pcmd);
 }
 
-void MultiCommands::Insert(Command *pcmd)
+void MultiCommands::Insert(Command *pcmd, size_t ioffset)
 {
-  // VERY INEFFICIENT - use sparingly to add commands at the front of the
+  // VERY INEFFICIENT - use sparingly to insert commands into the
   // multi-command vector
   ASSERT(pcmd != NULL);
-  m_vpcmds.insert(m_vpcmds.begin(), pcmd);
-}
-
-bool MultiCommands::Remove(Command *pcmd)
-{
-  ASSERT(pcmd != NULL);
-  std::vector<Command *>::iterator cmd_Iter;
-
-  cmd_Iter = find(m_vpcmds.begin(), m_vpcmds.end(), pcmd);
-  if (cmd_Iter != m_vpcmds.end()) {
-    delete (*cmd_Iter);
-    m_vpcmds.erase(cmd_Iter);
-    return true;
-  } else
-    return false;
-}
-
-bool MultiCommands::Remove()
-{
-  if (!m_vpcmds.empty()) {
-    delete m_vpcmds.back();
-    m_vpcmds.pop_back();
-    return true;
-  } else
-    return false;
+  pcmd->SetInMultiCommand();
+  m_vpcmds.insert(m_vpcmds.begin() + ioffset, pcmd);
 }
 
 bool MultiCommands::GetRC(Command *pcmd, int &rc)
@@ -194,8 +221,9 @@ bool MultiCommands::GetRC(const size_t ncmd, int &rc)
 // ------------------------------------------------
 
 UpdateGUICommand::UpdateGUICommand(CommandInterface *pcomInt,
-                                   ExecuteFn When, GUI_Action ga)
-  : Command(pcomInt), m_When(When), m_ga(ga)
+                                   ExecuteFn When, GUI_Action ga,
+                                   const pws_os::CUUID &entryUUID)
+ : Command(pcomInt), m_When(When), m_ga(ga), m_entryUUID(entryUUID)
 {
 }
 
@@ -203,7 +231,7 @@ int UpdateGUICommand::Execute()
 {
   if (m_When == WN_EXECUTE || m_When == WN_EXECUTE_REDO ||
       m_When == WN_REDO || m_When == WN_ALL) {
-    m_pcomInt->NotifyGUINeedsUpdating(m_ga, CUUID::NullUUID());
+    m_pcomInt->NotifyGUINeedsUpdating(m_ga, m_entryUUID);
   }
   return 0;
 }
@@ -211,7 +239,7 @@ int UpdateGUICommand::Execute()
 void UpdateGUICommand::Undo()
 {
   if (m_When == WN_UNDO || m_When == WN_ALL) {
-    m_pcomInt->NotifyGUINeedsUpdating(m_ga, CUUID::NullUUID());
+    m_pcomInt->NotifyGUINeedsUpdating(m_ga, m_entryUUID);
   }
 }
 
@@ -219,23 +247,29 @@ void UpdateGUICommand::Undo()
 // DBPrefsCommand
 // ------------------------------------------------
 
-DBPrefsCommand::DBPrefsCommand(CommandInterface *pcomInt, StringX &sxDBPrefs)
-  : Command(pcomInt), m_sxNewDBPrefs(sxDBPrefs)
+DBPrefsCommand::DBPrefsCommand(CommandInterface *pcomInt, StringX &sxDBPrefs, uint32 newHashIters)
+  : Command(pcomInt),
+    m_sxOldDBPrefs(PWSprefs::GetInstance()->Store()),
+    m_sxNewDBPrefs(sxDBPrefs),
+    m_oldHashIters(pcomInt->GetHashIters()),
+    m_newHashIters(newHashIters)
 {
-  m_sxOldDBPrefs = PWSprefs::GetInstance()->Store();
 }
 
 int DBPrefsCommand::Execute()
 {
-  PWSprefs::GetInstance()->Load(m_sxNewDBPrefs);
-  if (!m_pcomInt->IsReadOnly())
-    m_pcomInt->SetDBPrefsChanged(m_pcomInt->HaveHeaderPreferencesChanged(m_sxNewDBPrefs)); //?????
-
-  if (m_bNotifyGUI) {
-    m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_DB_PREFERENCES_CHANGED,
-                                      CUUID::NullUUID());
+  if (!m_pcomInt->IsReadOnly()) {
+    PWSprefs::GetInstance()->Load(m_sxNewDBPrefs);
+    // m_newHashIters may be zero if command created only with pref string,
+    // in places where the value was certainly unchanged.
+    if (m_newHashIters != 0)
+      m_pcomInt->SetHashIters(m_newHashIters);
+    if (m_bNotifyGUI) {
+      m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_DB_PREFERENCES_CHANGED,
+        CUUID::NullUUID());
+    }
+    m_CommandDBChange = DBPREFS;
   }
-  m_CommandDBChange = DBPREFS;
   return 0;
 }
 
@@ -243,10 +277,11 @@ void DBPrefsCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DBPREFS) {
     PWSprefs::GetInstance()->Load(m_sxOldDBPrefs);
+    m_pcomInt->SetHashIters(m_oldHashIters);
 
     if (m_bNotifyGUI) {
       m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_DB_PREFERENCES_CHANGED,
-        CUUID::NullUUID());
+                                        CUUID::NullUUID());
     }
   }
 }
@@ -347,19 +382,19 @@ DBEmptyGroupsCommand::DBEmptyGroupsCommand(CommandInterface *pcomInt,
   m_vOldEmptyGroups = pcomInt->GetEmptyGroups();
 }
 
-
 DBEmptyGroupsCommand::DBEmptyGroupsCommand(CommandInterface *pcomInt,
                                            const StringX &sxOldGroup,
                                            const StringX &sxNewGroup,
                                            Function function)
   : Command(pcomInt), m_sxOldGroup(sxOldGroup), m_sxNewGroup(sxNewGroup),
-  m_function(function), m_bSingleGroup(function == EG_RENAME)
+  m_function(function)
 {
   // This function call is used to rename a single empty group (EG_RENAME) or
   // to rename all empty groups that are sub-groups of the current group
   // (EG_RENAMEPATH).
-}
 
+  m_bSingleGroup = (function == EG_ADD || function == EG_DELETE || function == EG_RENAME);
+}
 
 int DBEmptyGroupsCommand::Execute()
 {
@@ -414,7 +449,7 @@ int DBEmptyGroupsCommand::Execute()
     if (bChanged) {
       if (m_bNotifyGUI) {
         m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_REFRESH_TREE,
-          CUUID::NullUUID());
+                                          CUUID::NullUUID());
 
         m_CommandDBChange = DBEMPTYGROUP;
       }
@@ -475,6 +510,8 @@ AddEntryCommand::AddEntryCommand(CommandInterface *pcomInt, const CItemData &ci,
                                  const CItemAtt *att, const Command *pcmd)
   : Command(pcomInt), m_ci(ci)
 {
+  m_CommandChangeType = DB;
+
   if (att != NULL)
     m_att = *att;
 
@@ -494,17 +531,19 @@ AddEntryCommand::~AddEntryCommand()
 int AddEntryCommand::Execute()
 {
   if (!m_pcomInt->IsReadOnly()) {
+    SaveDBInformation();
+
     m_pcomInt->DoAddEntry(m_ci, &m_att);
     m_pcomInt->AddChangedNodes(m_ci.GetGroup());
 
     if (m_ci.IsDependent()) {
       m_pcomInt->DoAddDependentEntry(m_ci.GetBaseUUID(), m_ci.GetUUID(),
-        m_ci.GetEntryType());
+                                     m_ci.GetEntryType());
     }
 
     if (m_bNotifyGUI) {
       m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_ADD_ENTRY,
-        m_ci.GetUUID());
+                                        m_ci.GetUUID());
     }
 
     time_t tttXTime;
@@ -520,13 +559,16 @@ int AddEntryCommand::Execute()
 void AddEntryCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
-    DeleteEntryCommand dec(m_pcomInt, m_ci, this);
-    dec.Execute();
-
+    // Do actions in reverse order to Execute
     if (m_ci.IsDependent()) {
       m_pcomInt->DoRemoveDependentEntry(m_ci.GetBaseUUID(), m_ci.GetUUID(),
-        m_ci.GetEntryType());
+                                        m_ci.GetEntryType());
     }
+
+    DeleteEntryCommand delete_entry_cmd(m_pcomInt, m_ci, this);
+    delete_entry_cmd.Execute();
+
+    RestoreDBInformation();
   }
 }
 
@@ -536,8 +578,10 @@ void AddEntryCommand::Undo()
 
 DeleteEntryCommand::DeleteEntryCommand(CommandInterface *pcomInt,
                                        const CItemData &ci, const Command *pcmd)
-  : Command(pcomInt), m_ci(ci), m_dependents(0)
+  : Command(pcomInt), m_ci(ci)
 {
+  m_CommandChangeType = DB;
+
   if (pcmd != NULL) {
     m_bNotifyGUI = pcmd->GetGUINotify();
   }
@@ -568,7 +612,7 @@ DeleteEntryCommand::DeleteEntryCommand(CommandInterface *pcomInt,
         ItemListIter itemIter = pcomInt->Find(dep_uuid);
         ASSERT(itemIter != pcomInt->GetEntryEndIter());
         if (itemIter != pcomInt->GetEntryEndIter())
-          m_dependents.push_back(itemIter->second);
+          m_vdependents.push_back(itemIter->second);
       } // for all dependents
     } // IsBase
   } // !IsNormal
@@ -584,6 +628,8 @@ int DeleteEntryCommand::Execute()
   if (m_pcomInt->IsReadOnly())
     return 0;
 
+  SaveDBInformation();
+
   // There's at least one case where we may get here with an entry
   // that's already been deleted: Consider a base and its shortcut
   // both in the same group, and the group's being deleted:
@@ -614,42 +660,50 @@ int DeleteEntryCommand::Execute()
 void DeleteEntryCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
+    MultiCommands *pmulticmds = MultiCommands::Create(m_pcomInt);
+    pmulticmds->SetNested();
+
     if (m_ci.IsDependent()) {
       // Check if dep entry hasn't already been added - can happen if
       // base and dep in group that's being undeleted.
       if (m_pcomInt->Find(m_ci.GetUUID()) == m_pcomInt->GetEntryEndIter()) {
-        Command *pcmd = AddEntryCommand::Create(m_pcomInt, m_ci, m_ci.GetBaseUUID(), &m_att, this);
-        pcmd->Execute();
-        delete pcmd;
+        pmulticmds->Add(AddEntryCommand::Create(m_pcomInt, m_ci, m_ci.GetBaseUUID(), &m_att, this));
       }
     } else {
-      AddEntryCommand undo(m_pcomInt, m_ci, m_ci.GetBaseUUID(), &m_att, this);
-      undo.Execute();
+      pmulticmds->Add(AddEntryCommand::Create(m_pcomInt, m_ci, m_ci.GetUUID(), &m_att, this));
+
       if (m_ci.IsShortcutBase()) { // restore dependents
-        for (std::vector<CItemData>::iterator iter = m_dependents.begin();
-          iter != m_dependents.end(); iter++) {
-          Command *pcmd = AddEntryCommand::Create(m_pcomInt, *iter, iter->GetBaseUUID(), NULL);
-          pcmd->Execute();
-          delete pcmd;
+        for (std::vector<CItemData>::iterator iter = m_vdependents.begin();
+             iter != m_vdependents.end(); iter++) {
+          pmulticmds->Add(AddEntryCommand::Create(m_pcomInt, *iter, iter->GetBaseUUID(), NULL));
         }
       } else if (m_ci.IsAliasBase()) {
         // Undeleting an alias base means making all the dependents refer to the alias
         // again. Perhaps the easiest approach is to delete the existing entries
         // and create new aliases.
-        for (std::vector<CItemData>::iterator iter = m_dependents.begin();
-          iter != m_dependents.end(); iter++) {
+        for (std::vector<CItemData>::iterator iter = m_vdependents.begin();
+             iter != m_vdependents.end(); iter++) {
           // Need to check that alias still exists - could have been deleted in group along with item
           // being undone, in which case it will be added separately
           if (m_pcomInt->Find(iter->GetUUID()) == m_pcomInt->GetEntryEndIter())
             continue;
-          DeleteEntryCommand delExAlias(m_pcomInt, *iter, this);
-          delExAlias.Execute(); // out with the old...
-          Command *pcmd = AddEntryCommand::Create(m_pcomInt, *iter, iter->GetBaseUUID(), NULL, this);
-          pcmd->Execute(); // in with the new!
-          delete pcmd;
+
+          // out with the old...
+          pmulticmds->Add(DeleteEntryCommand::Create(m_pcomInt, *iter, this));
+          // in with the new!
+          pmulticmds->Add(AddEntryCommand::Create(m_pcomInt, *iter, iter->GetBaseUUID(), NULL, this));
         }
       } // IsAliasBase
     } // !IsDependent
+
+    // Now do everything, if there is anything to do.
+    if (!pmulticmds->IsEmpty())
+      pmulticmds->Execute();
+
+    // Since not needed for Undo/Redo again - delete it
+    delete pmulticmds;
+
+    RestoreDBInformation();
   } // R/W & change to undo
 }
 
@@ -665,6 +719,8 @@ EditEntryCommand::EditEntryCommand(CommandInterface *pcomInt,
   // We're only supposed to operate on entries
   // with same uuids, and possibly different fields
   ASSERT(m_old_ci.GetUUID() == m_new_ci.GetUUID());
+
+  m_CommandChangeType = DB;
 }
 
 EditEntryCommand::~EditEntryCommand()
@@ -674,6 +730,8 @@ EditEntryCommand::~EditEntryCommand()
 int EditEntryCommand::Execute()
 {
   if (!m_pcomInt->IsReadOnly()) {
+    SaveDBInformation();
+
     m_pcomInt->DoReplaceEntry(m_old_ci, m_new_ci);
 
     m_pcomInt->AddChangedNodes(m_old_ci.GetGroup());
@@ -703,9 +761,11 @@ void EditEntryCommand::Undo()
       // If the entry's group has changed, refresh the entire tree, otherwise, just the entry
       // in the tree and list views
       UpdateGUICommand::GUI_Action gac = (m_old_ci.GetGroup() != m_new_ci.GetGroup()) ?
-        UpdateGUICommand::GUI_REFRESH_TREE : UpdateGUICommand::GUI_REFRESH_ENTRY;
+                UpdateGUICommand::GUI_REFRESH_TREE : UpdateGUICommand::GUI_REFRESH_ENTRY;
       m_pcomInt->NotifyGUINeedsUpdating(gac, entry_uuid);
     }
+
+    RestoreDBInformation();
   }
 }
 
@@ -717,11 +777,11 @@ UpdateEntryCommand::UpdateEntryCommand(CommandInterface *pcomInt,
                                        const CItemData &ci,
                                        CItemData::FieldType ftype,
                                        const StringX &value)
-  : Command(pcomInt), m_ftype(ftype), m_value(value)
+  : Command(pcomInt), m_old_ci(ci), m_ftype(ftype)
 {
-  m_entry_uuid = ci.GetUUID();
-  m_old_status = ci.GetStatus();
-  m_old_value = ci.GetFieldValue(m_ftype);
+  m_CommandChangeType = DB;
+
+  m_new_ci.SetFieldValue(ftype, value);
 }
 
 void UpdateEntryCommand::Doit(const CUUID &entry_uuid,
@@ -735,14 +795,17 @@ void UpdateEntryCommand::Doit(const CUUID &entry_uuid,
     if (ftype != CItemData::PASSWORD)
       pos->second.SetFieldValue(ftype, value);
     else {
+      time_t tttoldXtime;
       if (efn == UpdateGUICommand::WN_EXECUTE_REDO) {
-        m_oldpwhistory = pos->second.GetPWHistory();
-        pos->second.GetXTime(m_tttoldXtime);
+        m_old_ci.SetPWHistory(pos->second.GetPWHistory());
+        pos->second.GetXTime(tttoldXtime);
+        m_old_ci.SetXTime(tttoldXtime);
         pos->second.UpdatePassword(value);
       } else {
         pos->second.SetPassword(value);
-        pos->second.SetXTime(m_tttoldXtime);
-        pos->second.SetPWHistory(m_oldpwhistory);
+        m_old_ci.GetXTime(tttoldXtime);
+        pos->second.SetXTime(tttoldXtime);
+        pos->second.SetPWHistory(m_old_ci.GetPWHistory());
       }
     }
     if (ftype == CItemData::PASSWORD ||
@@ -757,14 +820,18 @@ void UpdateEntryCommand::Doit(const CUUID &entry_uuid,
 int UpdateEntryCommand::Execute()
 {
   if (!m_pcomInt->IsReadOnly()) {
-    Doit(m_entry_uuid, m_ftype, m_value, CItemData::ES_MODIFIED, UpdateGUICommand::WN_EXECUTE_REDO);
+    SaveDBInformation();
+
+    Doit(m_old_ci.GetUUID(), m_ftype, m_new_ci.GetFieldValue(m_ftype),
+         CItemData::ES_MODIFIED, UpdateGUICommand::WN_EXECUTE_REDO);
 
     if (m_bNotifyGUI)
       m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_REFRESH_ENTRYFIELD,
-        m_entry_uuid, m_ftype);
+                                        m_old_ci.GetUUID(), m_ftype);
 
     if (m_ftype == CItemData::XTIME)
-      m_pcomInt->UpdateExpiryEntry(m_entry_uuid, m_ftype, m_value);
+      m_pcomInt->UpdateExpiryEntry(m_old_ci.GetUUID(), m_ftype,
+                                   m_new_ci.GetFieldValue(m_ftype));
 
     m_CommandDBChange = DB;
   }
@@ -774,11 +841,14 @@ int UpdateEntryCommand::Execute()
 void UpdateEntryCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
-    Doit(m_entry_uuid, m_ftype, m_old_value, m_old_status, UpdateGUICommand::WN_UNDO);
+    Doit(m_old_ci.GetUUID(), m_ftype, m_old_ci.GetFieldValue(m_ftype),
+         m_old_ci.GetStatus(), UpdateGUICommand::WN_UNDO);
 
     if (m_bNotifyGUI)
       m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_REFRESH_ENTRYFIELD,
-        m_entry_uuid, m_ftype);
+                                        m_old_ci.GetUUID(), m_ftype);
+  
+    RestoreDBInformation();
   }
 }
 
@@ -787,26 +857,27 @@ void UpdateEntryCommand::Undo()
 // ------------------------------------------------
 
 UpdatePasswordCommand::UpdatePasswordCommand(CommandInterface *pcomInt,
-                                             CItemData &ci,
-                                             const StringX sxNewPassword)
-  : Command(pcomInt), m_sxNewPassword(sxNewPassword)
+                                             const CItemData &ci,
+                                             const StringX &sxNewPassword)
+  : Command(pcomInt), m_old_ci(ci)
 {
-  m_entry_uuid = ci.GetUUID();
-  m_old_status = ci.GetStatus();
-  m_sxOldPassword = ci.GetPassword();
-  m_sxOldPWHistory = ci.GetPWHistory();
-  ci.GetXTime(m_tttOldXTime);
+  m_CommandChangeType = DB;
+
+  m_new_ci.SetPassword(sxNewPassword);
 }
 
 int UpdatePasswordCommand::Execute()
 {
-  if (!m_pcomInt->IsReadOnly() && m_sxOldPassword != m_sxNewPassword) {
-    ItemListIter pos = m_pcomInt->Find(m_entry_uuid);
+  if (!m_pcomInt->IsReadOnly() &&m_old_ci.GetPassword() != m_new_ci.GetPassword()) {
+    SaveDBInformation();
+
+    ItemListIter pos = m_pcomInt->Find(m_old_ci.GetUUID());
     if (pos != m_pcomInt->GetEntryEndIter()) {
-      pos->second.UpdatePassword(m_sxNewPassword);
-      time_t tttNewXTime;
+      pos->second.UpdatePassword(m_new_ci.GetPassword());
+      time_t tttNewXTime, tttOldXTime;
       pos->second.GetXTime(tttNewXTime);
-      if (m_tttOldXTime != tttNewXTime) {
+      m_old_ci.GetXTime(tttOldXTime);
+      if (tttOldXTime != tttNewXTime) {
         m_pcomInt->UpdateExpiryEntry(pos->second);
       }
       pos->second.SetStatus(CItemData::ES_MODIFIED);
@@ -817,8 +888,7 @@ int UpdatePasswordCommand::Execute()
 
     if (m_bNotifyGUI)
       m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_REFRESH_ENTRYPASSWORD,
-        m_entry_uuid);
-
+                                        m_old_ci.GetUUID());
   }
   return 0;
 }
@@ -826,17 +896,21 @@ int UpdatePasswordCommand::Execute()
 void UpdatePasswordCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
-    ItemListIter pos = m_pcomInt->Find(m_entry_uuid);
+    ItemListIter pos = m_pcomInt->Find(m_old_ci.GetUUID());
     if (pos != m_pcomInt->GetEntryEndIter()) {
-      pos->second.SetPassword(m_sxOldPassword);
-      pos->second.SetPWHistory(m_sxOldPWHistory);
-      pos->second.SetStatus(m_old_status);
-      pos->second.SetXTime(m_tttOldXTime);
+      time_t tttOldXTime;
+      m_old_ci.GetXTime(tttOldXTime);
+      pos->second.SetPassword(m_old_ci.GetPassword());
+      pos->second.SetPWHistory(m_old_ci.GetPWHistory());
+      pos->second.SetStatus(m_old_ci.GetStatus());
+      pos->second.SetXTime(tttOldXTime);
     }
 
     if (m_bNotifyGUI)
       m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_REFRESH_ENTRYPASSWORD,
-        m_entry_uuid);
+                                        m_old_ci.GetUUID());
+
+    RestoreDBInformation();  
   }
 }
 
@@ -851,11 +925,14 @@ AddDependentEntryCommand::AddDependentEntryCommand(CommandInterface *pcomInt,
   : Command(pcomInt), m_base_uuid(base_uuid),
     m_entry_uuid(entry_uuid), m_type(type)
 {
+  m_CommandChangeType = DB;
 }
 
 int AddDependentEntryCommand::Execute()
 {
   if (!m_pcomInt->IsReadOnly()) {
+    SaveDBInformation();
+
     m_pcomInt->DoAddDependentEntry(m_base_uuid, m_entry_uuid, m_type);
 
     m_CommandDBChange = DB;
@@ -867,6 +944,8 @@ void AddDependentEntryCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
     m_pcomInt->DoRemoveDependentEntry(m_base_uuid, m_entry_uuid, m_type);
+  
+    RestoreDBInformation();
   }
 }
 
@@ -882,20 +961,19 @@ AddDependentEntriesCommand::AddDependentEntriesCommand(CommandInterface *pcomInt
   : Command(pcomInt), m_dependentslist(dependentslist), m_pRpt(pRpt),
     m_type(type), m_iVia(iVia)
 {
-  m_pmapDeletedItems = new ItemList;
-  m_pmapSaveStatus = new SaveTypePWMap;
+  m_CommandChangeType = DB;
 }
 
 AddDependentEntriesCommand::~AddDependentEntriesCommand()
 {
-  delete m_pmapDeletedItems;
-  delete m_pmapSaveStatus;
 }
 
 int AddDependentEntriesCommand::Execute()
 {
   int rc(0);
   if (!m_pcomInt->IsReadOnly()) {
+    SaveDBInformation();
+
     if (m_type == CItemData::ET_ALIAS) {
       m_saved_base2aliases_mmap = m_pcomInt->GetBase2AliasesMmap();
     } else { // if !alias, assume shortcut
@@ -903,8 +981,8 @@ int AddDependentEntriesCommand::Execute()
     }
 
     rc = m_pcomInt->DoAddDependentEntries(m_dependentslist, m_pRpt,
-      m_type, m_iVia,
-      m_pmapDeletedItems, m_pmapSaveStatus);
+                                          m_type, m_iVia,
+                                          &m_mapDeletedItems, &m_mapSaveStatus);
 
     m_CommandDBChange = DB;
   }
@@ -914,13 +992,15 @@ int AddDependentEntriesCommand::Execute()
 void AddDependentEntriesCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
-    m_pcomInt->UndoAddDependentEntries(m_pmapDeletedItems, m_pmapSaveStatus);
+    m_pcomInt->UndoAddDependentEntries(&m_mapDeletedItems, &m_mapSaveStatus);
 
     if (m_type == CItemData::ET_ALIAS) {
       m_pcomInt->SetBase2AliasesMmap(m_saved_base2aliases_mmap);
     } else { // if !alias, assume shortcut
       m_pcomInt->SetBase2ShortcutsMmap(m_saved_base2shortcuts_mmap);
     }
+
+    RestoreDBInformation();
   }
 }
 
@@ -935,11 +1015,14 @@ RemoveDependentEntryCommand::RemoveDependentEntryCommand(CommandInterface *pcomI
   : Command(pcomInt), m_base_uuid(base_uuid),
     m_entry_uuid(entry_uuid), m_type(type)
 {
+  m_CommandChangeType = DB;
 }
 
 int RemoveDependentEntryCommand::Execute()
 {
   if (!m_pcomInt->IsReadOnly()) {
+    SaveDBInformation();
+
     m_pcomInt->DoRemoveDependentEntry(m_base_uuid, m_entry_uuid, m_type);
 
     m_CommandDBChange = DB;
@@ -951,6 +1034,8 @@ void RemoveDependentEntryCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
     m_pcomInt->DoAddDependentEntry(m_base_uuid, m_entry_uuid, m_type);
+  
+    RestoreDBInformation();
   }
 }
 
@@ -965,11 +1050,20 @@ MoveDependentEntriesCommand::MoveDependentEntriesCommand(CommandInterface *pcomI
   : Command(pcomInt), m_from_baseuuid(from_baseuuid),
     m_to_baseuuid(to_baseuuid), m_type(type)
 {
+  m_CommandChangeType = DB;
 }
 
 int MoveDependentEntriesCommand::Execute()
 {
   if (!m_pcomInt->IsReadOnly()) {
+    SaveDBInformation();
+
+    if (m_type == CItemData::ET_ALIAS) {
+      m_saved_base2aliases_mmap = m_pcomInt->GetBase2AliasesMmap();
+    } else { // if !alias, assume shortcut
+      m_saved_base2shortcuts_mmap = m_pcomInt->GetBase2ShortcutsMmap();
+    }
+
     if (m_pcomInt->DoMoveDependentEntries(m_from_baseuuid, m_to_baseuuid, m_type))
       m_CommandDBChange = DB;
   }
@@ -978,8 +1072,16 @@ int MoveDependentEntriesCommand::Execute()
 
 void MoveDependentEntriesCommand::Undo()
 {
+  // Can't move all back by calling DoMoveDependentEntries with reversed arguments
+  // as there may have been some originally linked that should not be moved.
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
-    m_pcomInt->DoMoveDependentEntries(m_to_baseuuid, m_from_baseuuid, m_type);
+    if (m_type == CItemData::ET_ALIAS) {
+      m_pcomInt->SetBase2AliasesMmap(m_saved_base2aliases_mmap);
+    } else { // if !alias, assume shortcut
+      m_pcomInt->SetBase2ShortcutsMmap(m_saved_base2shortcuts_mmap);
+    }
+  
+    RestoreDBInformation();
   }
 }
 
@@ -991,14 +1093,18 @@ UpdatePasswordHistoryCommand::UpdatePasswordHistoryCommand(CommandInterface *pco
                                                            const int iAction,
                                                            const int new_default_max)
  : Command(pcomInt), m_iAction(iAction), m_new_default_max(new_default_max)
-{}
+{
+  m_CommandChangeType = DB;
+}
 
 int UpdatePasswordHistoryCommand::Execute()
 {
   int rc(0);
   if (!m_pcomInt->IsReadOnly()) {
+    SaveDBInformation();
+
     rc = m_pcomInt->DoUpdatePasswordHistory(m_iAction, m_new_default_max,
-                               m_mapSavedHistory);
+                                            m_mapSavedHistory);
 
     m_CommandDBChange = DB;
   }
@@ -1009,6 +1115,8 @@ void UpdatePasswordHistoryCommand::Undo()
 {
   if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
     m_pcomInt->UndoUpdatePasswordHistory(m_mapSavedHistory);
+  
+    RestoreDBInformation();
   }
 }
 
@@ -1018,14 +1126,30 @@ void UpdatePasswordHistoryCommand::Undo()
 
 RenameGroupCommand::RenameGroupCommand(CommandInterface *pcomInt,
                                        const StringX sxOldPath, const StringX sxNewPath)
- : Command(pcomInt), m_sxOldPath(sxOldPath), m_sxNewPath(sxNewPath)
-{}
+ : Command(pcomInt), m_sxOldPath(sxOldPath), m_sxNewPath(sxNewPath), m_pmulticmds(NULL)
+{
+  m_CommandChangeType = DB;
+}
+
+RenameGroupCommand::~RenameGroupCommand()
+{
+  delete m_pmulticmds;
+}
 
 int RenameGroupCommand::Execute()
 {
   int rc(0);
   if (!m_pcomInt->IsReadOnly() && m_sxOldPath != m_sxNewPath) {
-    rc = m_pcomInt->DoRenameGroup(m_sxOldPath, m_sxNewPath);
+    SaveDBInformation();
+
+    if (m_pmulticmds == NULL) {
+      // Execute (will not be NULL if performing a Redo)
+      rc = m_pcomInt->DoRenameGroup(m_sxOldPath, m_sxNewPath, m_pmulticmds);
+    }
+
+    // Do it
+    if (!m_pmulticmds->IsEmpty())
+      m_pmulticmds->Execute();
 
     m_CommandDBChange = DB;
   }
@@ -1034,8 +1158,11 @@ int RenameGroupCommand::Execute()
 
 void RenameGroupCommand::Undo()
 {
-  if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB) {
-    m_pcomInt->UndoRenameGroup(m_sxOldPath, m_sxNewPath);
+  if (!m_pcomInt->IsReadOnly() && m_CommandDBChange == DB &&
+      !m_pmulticmds->IsEmpty()) {
+    m_pcomInt->UndoRenameGroup(m_pmulticmds);
+  
+    RestoreDBInformation();
   }
 }
 
@@ -1090,7 +1217,7 @@ int DBFiltersCommand::Execute()
 
       if (m_bNotifyGUI) {
         m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_UPDATE_STATUSBAR,
-          CUUID::NullUUID());
+                                          CUUID::NullUUID());
       }
     }
   }
@@ -1104,8 +1231,7 @@ void DBFiltersCommand::Undo()
 
     if (m_bNotifyGUI) {
       m_pcomInt->NotifyGUINeedsUpdating(UpdateGUICommand::GUI_UPDATE_STATUSBAR,
-        CUUID::NullUUID());
+                                        CUUID::NullUUID());
     }
   }
 }
-
