@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -23,6 +23,7 @@ class CommandInterface;
 
 #include <map>
 #include <vector>
+#include <typeinfo>
 
 /**
  * Command-derived classes are used to support undo/redo.
@@ -32,6 +33,8 @@ class CommandInterface;
  * no Command object can be created on the stack.
  */
 
+class MultiCommands;
+
 // Base Command class
 
 class Command
@@ -40,8 +43,6 @@ public:
   enum CommandDBChange { NONE = -1, MULTICOMMAND, DB, DBPREFS, DBHEADER, DBEMPTYGROUP, DBPOLICYNAMES,
     DBFILTERS };
 
-  enum StateType { COMMANDACTION = 0, PREEXECUTE, POSTEXECUTE };
-
   virtual ~Command();
   virtual int Execute() = 0;
   virtual int Redo() {return Execute();} // common case
@@ -50,28 +51,43 @@ public:
   void SetNoGUINotify() {m_bNotifyGUI = false;}
   bool GetGUINotify() const {return m_bNotifyGUI;}
 
-  void SaveChangedState(StateType st, st_DBChangeStatus &stDBCS);
-  void RestoreChangedState(st_DBChangeStatus &stDBCS);
+  // This tells a MultiCommand that this command could change an entry
+  // as opposed to a DB preference, header, empty group, password policy or filter
+  bool IsEntryChangeType() const { return m_CommandChangeType == DB; }
 
-  CommandDBChange GetCommandChange() const { return m_CommandDBChange; }
+  // This states if something was actually changed
+  bool WasDBChanged() const { return m_CommandDBChange != NONE; }
 
-  const st_DBChangeStatus &GetPostCommandStatus() const
-  { return m_PostCommand; }
-
-  const st_DBChangeStatus &GetCommandStatus() const
-  { return m_Command; }
-  
 protected:
   Command(CommandInterface *pcomInt); // protected constructor!
 
+  void SaveDBInformation();
+  void RestoreDBInformation();
+
+  // If a command is within a MultiCommand, do not save DB information
+  // other than that needed for the actual command.
+  // This prevents multiple copies of similar data that is only needed once
+  // in a MultiCommand i.e. at creation and during Undo
+  bool InMultiCommand() const { return m_bInMultiCommand; }
+
+  friend class MultiCommands; // Yes, a derived class needs to be a friend here...
+  void SetInMultiCommand() { m_bInMultiCommand = true; }
+  
   CommandInterface *m_pcomInt;
-  bool m_bNotifyGUI;
-
-  st_DBChangeStatus m_PreCommand;
-  st_DBChangeStatus m_PostCommand;
-  st_DBChangeStatus m_Command;
+  bool m_bNotifyGUI, m_bInMultiCommand;
 
+  // Command return code (not all commands set this to anything other than zero)
   int m_RC;
+
+  // If needed, to be used by a Multicommand that changes the DB or
+  // a single command that changes the DB outside a MultiCommand
+  std::vector<StringX> m_vSavedModifiedNodes;
+
+  // This is the potential change type if it there is anything to do
+  // It is set so that MultiCommands can tell whether to save DB information
+  CommandDBChange m_CommandChangeType;
+
+  // The command change value is ONLY set during Execute
   CommandDBChange m_CommandDBChange;
 };
 
@@ -106,22 +122,27 @@ public:
     GUI_UNDO_MERGESYNC,
     GUI_REFRESH_TREE,
     GUI_REFRESH_ENTRY,
+    GUI_REFRESH_GROUPS,
+    GUI_REFRESH_BOTHVIEWS,
     GUI_DB_PREFERENCES_CHANGED,
     GUI_PWH_CHANGED_IN_DB
   };
 
+  // Note: entryUUID only used in GUI_REFRESH_ENTRY
   static UpdateGUICommand *Create(CommandInterface *pcomInt,
-                                  ExecuteFn When, GUI_Action ga)
-  { return new UpdateGUICommand(pcomInt, When, ga); }
+                                  ExecuteFn When, GUI_Action ga,
+                                  const pws_os::CUUID &entryUUID = pws_os::CUUID::NullUUID())
+  { return new UpdateGUICommand(pcomInt, When, ga, entryUUID); }
   int Execute();
   void Undo();
 
 private:
   UpdateGUICommand& operator=(const UpdateGUICommand&); // Do not implement
   UpdateGUICommand(CommandInterface *pcomInt, ExecuteFn When,
-                   GUI_Action ga);
+                   GUI_Action ga, const pws_os::CUUID &entryUUID);
   const ExecuteFn m_When;
   const GUI_Action m_ga;
+  pws_os::CUUID m_entryUUID;
 };
 
 // PWS related commands
@@ -129,16 +150,17 @@ private:
 class DBPrefsCommand : public Command
 {
 public:
+  // Call Create without newHashIters when this hasn't changed.
   static DBPrefsCommand *Create(CommandInterface *pcomInt,
-                                StringX &sxNewDBPrefs)
-  { return new DBPrefsCommand(pcomInt, sxNewDBPrefs); }
+                                StringX &sxNewDBPrefs, uint32 newHashIters = 0)
+  { return new DBPrefsCommand(pcomInt, sxNewDBPrefs, newHashIters); }
   int Execute();
   void Undo();
 
 private:
-  DBPrefsCommand(CommandInterface *pcomInt, StringX &sxNewDBPrefs);
-  StringX m_sxOldDBPrefs;
-  StringX m_sxNewDBPrefs;
+  DBPrefsCommand(CommandInterface *pcomInt, StringX &sxNewDBPrefs, uint32 newHashIters);
+  const StringX m_sxOldDBPrefs, m_sxNewDBPrefs;
+  const uint32 m_oldHashIters, m_newHashIters;
 };
 
 class DBPolicyNamesCommand : public Command
@@ -248,7 +270,7 @@ private:
   const CItemData m_ci;
   CItemAtt m_att;
   pws_os::CUUID m_base_uuid; // for undo of shortcut or alias deletion
-  std::vector<CItemData> m_dependents; // for undo of base deletion
+  std::vector<CItemData> m_vdependents; // for undo of base deletion
 };
 
 class EditEntryCommand : public Command
@@ -290,11 +312,9 @@ private:
             CItemData::EntryStatus es,
             UpdateGUICommand::ExecuteFn efn);
 
-  pws_os::CUUID m_entry_uuid;
-  CItemData::FieldType m_ftype;
-  StringX m_value, m_old_value, m_oldpwhistory;
-  time_t m_tttoldXtime;
-  CItemData::EntryStatus m_old_status;
+  CItemData m_old_ci;
+  CItemData m_new_ci;
+  const CItemData::FieldType m_ftype;
 };
 
 class UpdatePasswordCommand : public Command
@@ -309,11 +329,9 @@ public:
 
 private:
   UpdatePasswordCommand(CommandInterface *pcomInt,
-                        CItemData &ci, const StringX sxNewPassword);
-  pws_os::CUUID m_entry_uuid;
-  StringX m_sxNewPassword, m_sxOldPassword, m_sxOldPWHistory;
-  CItemData::EntryStatus m_old_status;
-  time_t m_tttOldXTime;
+                        const CItemData &ci, const StringX &sxNewPassword);
+  const CItemData m_old_ci; // for uuid, password, password history, XTime, entry status
+  CItemData m_new_ci; // for new password
 };
 
 class AddDependentEntryCommand : public Command
@@ -356,11 +374,12 @@ private:
                              UUIDVector &dependentslist, CReport *pRpt,
                              CItemData::EntryType type, int iVia);
   UUIDVector m_dependentslist;
-  ItemList *m_pmapDeletedItems;
-  SaveTypePWMap *m_pmapSaveStatus;
+  ItemList m_mapDeletedItems;
+  SaveTypePWMap m_mapSaveStatus;
   CReport *m_pRpt;
   CItemData::EntryType m_type;
   int m_iVia;
+
   // Alias/Shortcut structures
   // Permanent Multimap: since potentially more than one alias/shortcut per base
   //  Key = base uuid; Value = multiple alias/shortcut uuids
@@ -410,6 +429,9 @@ private:
   pws_os::CUUID m_from_baseuuid;
   pws_os::CUUID m_to_baseuuid;
   CItemData::EntryType m_type;
+
+  ItemMMap m_saved_base2aliases_mmap;
+  ItemMMap m_saved_base2shortcuts_mmap;
 };
 
 class UpdatePasswordHistoryCommand : public Command
@@ -437,14 +459,19 @@ public:
   static RenameGroupCommand *Create(CommandInterface *pcomInt,
                                     const StringX sxOldPath, const StringX sxNewPath)
   { return new RenameGroupCommand(pcomInt, sxOldPath, sxNewPath); }
+  ~RenameGroupCommand();
   int Execute();
   void Undo();
 
+  void GetPaths(StringX &sxOldPath, StringX &sxNewPath) const
+  { sxOldPath = m_sxOldPath; sxNewPath = m_sxNewPath; }
+
 private:
   RenameGroupCommand(CommandInterface *pcomInt,
                      StringX sxOldPath, StringX sxNewPath);
 
    StringX m_sxOldPath, m_sxNewPath;
+   MultiCommands *m_pmulticmds;
 };
 
 class ChangeDBHeaderCommand : public Command {
@@ -490,17 +517,19 @@ public:
   void Undo();
 
   void Add(Command *pcmd);
-  void Insert(Command *pcmd); // VERY INEFFICIENT - use sparingly
-  bool Remove(Command *pcmd);
-  bool Remove();
+  void Insert(Command *pcmd, size_t ioffset = 0); // VERY INEFFICIENT - use sparingly
   bool GetRC(Command *pcmd, int &rc);
   bool GetRC(const size_t ncmd, int &rc);
   std::size_t GetSize() const {return m_vpcmds.size();}
+  bool IsEmpty() const { return m_vpcmds.empty(); }
+  void SetNested() { SetInMultiCommand(); }
 
-  std::vector<Command *> m_vpcmds;
+  Command *FindCommand(const std::type_info &ti);
 
  private:
   MultiCommands(CommandInterface *pcomInt);
+
+  std::vector<Command *> m_vpcmds;
   std::vector<int> m_vRCs;
 };
 
