@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -872,6 +872,7 @@ int PWScore::MergeDependents(PWScore *pothercore, MultiCommands *pmulticmds,
                           sx_merged.c_str(), str_timestring.c_str());
       ci_temp.SetTitle(sx_newTitle);
     }
+
     // Check this is unique - if not - don't add this one! - its only an alias/shortcut!
     // We can't keep trying for uniqueness after adding a timestamp!
     foundPos = Find(ci_temp.GetGroup(), sx_newTitle, ci_temp.GetUser());
@@ -879,7 +880,8 @@ int PWScore::MergeDependents(PWScore *pothercore, MultiCommands *pmulticmds,
       continue;
 
     ci_temp.SetBaseUUID(new_base_uuid);
-    Command *pcmd1 = AddEntryCommand::Create(this, ci_temp);
+    ci_temp.SetStatus(CItemData::ES_ADDED);
+    Command *pcmd1 = AddEntryCommand::Create(this, ci_temp, new_base_uuid);
     pcmd1->SetNoGUINotify();
     pmulticmds->Add(pcmd1);
 
@@ -928,7 +930,7 @@ void PWScore::Synchronize(PWScore *pothercore,
     CItemData::BASEUUID, CItemData::ALIASUUID, CItemData::SHORTCUTUUID };
 
   // Turn them off
-  for (int i = 0; i < sizeof(ftInappropriateSyncFields) / sizeof(CItemData::FieldType); i++) {
+  for (size_t i = 0; i < sizeof(ftInappropriateSyncFields) / sizeof(CItemData::FieldType); i++) {
     bsSyncFields.reset(ftInappropriateSyncFields[i]);
   }
 
@@ -988,7 +990,6 @@ void PWScore::Synchronize(PWScore *pothercore,
         continue;
 
       CItemData updItem(curItem);
-      updItem.SetDisplayInfo(NULL);
 
       if (curItem.GetUUID() != otherItem.GetUUID()) {
         pws_os::Trace(_T("Synchronize: Mis-match UUIDs for [%ls:%ls:%ls]\n"),
@@ -1021,7 +1022,6 @@ void PWScore::Synchronize(PWScore *pothercore,
       if (!bUpdated)
         continue;
 
-      GUISetupDisplayInfo(updItem);
       updItem.SetStatus(CItemData::ES_MODIFIED);
 
       StringX sx_updated;
