@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -555,14 +555,14 @@ struct XMLFilterWriterToString {
   {}
 
   // operator
-  void operator()(pair<const st_Filterkey, st_filters> p)
+  void operator()(const pair<const st_Filterkey, st_filters> &p)
   {
     string xml = GetFilterXML(p.second, m_bWithFormatting);
     m_os << xml.c_str();
   }
 
 private:
-  XMLFilterWriterToString& operator=(const XMLFilterWriterToString&); // Do not implement
+  XMLFilterWriterToString& operator=(const XMLFilterWriterToString&) = delete;
   coStringXStream &m_os;
   bool m_bWithFormatting;
 };
@@ -774,7 +774,7 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
         Format(cs_criteria, L"%ls", cs_rule.c_str());
       else {
         stringT cs_delim(L"");
-        if (cs1.find(L" ") != stringT::npos)
+        if (cs1.find(L' ') != stringT::npos)
           cs_delim = L"'";
         Format(cs_criteria, L"%ls %ls%ls%ls %ls", 
                cs_rule.c_str(), cs_delim.c_str(), 
@@ -872,8 +872,8 @@ PWSFilterManager::PWSFilterManager()
     fr.fdate1 = 0;
     m_expirefilter.vMfldata.push_back(fr);
     m_expirefilter.num_Mactive = (int)m_expirefilter.vMfldata.size();
-
   }
+
   {
     LoadAString(m_unsavedfilter.fname, IDSC_NONSAVEDCHANGES);
 
@@ -891,6 +891,25 @@ PWSFilterManager::PWSFilterManager()
     m_unsavedfilter.vMfldata.push_back(fr);
     m_unsavedfilter.num_Mactive = (int)m_unsavedfilter.vMfldata.size();
   }
+
+  {
+    LoadAString(m_lastfoundfilter.fname, IDSC_FOUNDENTRIESFILTER);
+
+    // Actual values not needed as matching done by UUID
+    st_FilterRow fr;
+
+    fr.bFilterComplete = true;
+    fr.ftype = FT_INVALID;
+    fr.mtype = PWSMatch::MT_INVALID;
+    fr.rule = PWSMatch::MR_INVALID;
+    fr.ltype = LC_OR;
+
+    fr.fdate1 = 0;
+    m_lastfoundfilter.vMfldata.push_back(fr);
+    m_lastfoundfilter.num_Mactive = (int)m_lastfoundfilter.vMfldata.size();
+  }
+
+  m_bFindFilterActive = false;
 }
 
 void PWSFilterManager::CreateGroups()
@@ -1038,6 +1057,14 @@ void PWSFilterManager::CreateGroups()
     m_vAflgroups.clear();
 }
 
+void PWSFilterManager::SetFilterFindEntries(std::vector<pws_os::CUUID> *pvFoundUUIDs)
+{
+  if (pvFoundUUIDs == NULL)
+    m_vFltrFoundUUIDs.clear();
+  else
+    m_vFltrFoundUUIDs = *pvFoundUUIDs;
+}
+
 bool PWSFilterManager::PassesFiltering(const CItemData &ci, const PWScore &core)
 {
   bool thistest_rc;
@@ -1048,6 +1075,11 @@ bool PWSFilterManager::PassesFiltering(const CItemData &ci, const PWScore &core)
   if (!m_currentfilter.IsActive())
     return true;
 
+  if (m_bFindFilterActive) {
+    return (std::find(m_vFltrFoundUUIDs.begin(), m_vFltrFoundUUIDs.end(),
+                      ci.GetUUID()) != m_vFltrFoundUUIDs.end());
+  }
+
   const CItemData::EntryType entrytype = ci.GetEntryType();
 
   for (auto groups_iter = m_vMflgroups.begin();
@@ -1066,8 +1098,8 @@ bool PWSFilterManager::PassesFiltering(const CItemData &ci, const PWScore &core)
       thistest_rc = false;
 
       PWSMatch::MatchType mt(PWSMatch::MT_INVALID);
-      const FieldType ft = m_currentfilter.vMfldata[num].ftype;
-      const int ifunction = (int)st_fldata.rule;
+      const FieldType ft = st_fldata.ftype;
+      const auto ifunction = (int)st_fldata.rule;
 
       switch (ft) {
         case FT_GROUPTITLE:
@@ -1197,7 +1229,7 @@ bool PWSFilterManager::PassesFiltering(const CItemData &ci, const PWScore &core)
             if (ifunction == PWSMatch::MR_BETWEEN)
               t2 = now + (st_fldata.fnum2 * 86400);
           }
-          thistest_rc = pci->Matches(t1, t2,
+          thistest_rc = pci->MatchesTime(t1, t2,
                                      (int)ft, ifunction);
           tests++;
           break;
@@ -1263,6 +1295,58 @@ bool PWSFilterManager::PassesFiltering(const CItemData &ci, const PWScore &core)
   return false;
 }
 
+bool PWSFilterManager::PassesEmptyGroupFiltering(const StringX &sxGroup)
+{
+  bool thistest_rc;
+
+  if (!m_currentfilter.IsActive())
+    return true;
+
+  for (auto groups_iter = m_vMflgroups.begin();
+       groups_iter != m_vMflgroups.end(); groups_iter++) {
+    const vfiltergroup &group = *groups_iter;
+
+    int tests(0);
+    bool thisgroup_rc = false;
+    for (auto iter = group.begin();
+         iter != group.end(); iter++) {
+      const int &num = *iter;
+      if (num == -1) // Padding to ensure group size is correct for FT_PWHIST & FT_POLICY
+        continue;
+
+      const st_FilterRow &st_fldata = m_currentfilter.vMfldata.at(num);
+      thistest_rc = false;
+
+      const FieldType ft = m_currentfilter.vMfldata[num].ftype;
+      const auto ifunction = (int)st_fldata.rule;
+
+      // We are only testing the group value and, as an empty group, it must be present
+      if (ft != FT_GROUP || ifunction == PWSMatch::MR_PRESENT || ifunction == PWSMatch::MR_NOTPRESENT) {
+        continue;
+      }
+
+      thistest_rc = PWSMatch::Match(st_fldata.fstring, sxGroup,
+                                    st_fldata.fcase ? -ifunction : ifunction);
+      tests++;   
+
+      if (tests <= 1)
+        thisgroup_rc = thistest_rc;
+      else {
+        //Within groups, tests are always "AND" connected
+        thisgroup_rc = thistest_rc && thisgroup_rc;
+      }
+    }
+
+    // This group of tests completed -
+    //   if 'thisgroup_rc == true', leave now; else go on to next group
+    if (thisgroup_rc)
+      return true;
+  }
+
+  // We finished all the groups and haven't found one that is true - exclude entry.
+  return false;
+}
+
 bool PWSFilterManager::PassesPWHFiltering(const CItemData *pci) const
 {
   bool thistest_rc, bPresent;
@@ -1302,7 +1386,7 @@ bool PWSFilterManager::PassesPWHFiltering(const CItemData *pci) const
           mt = PWSMatch::MT_BOOL;
           break;
         case HT_ACTIVE:
-          bValue = status == TRUE;
+          bValue = status;
           mt = PWSMatch::MT_BOOL;
           break;
         case HT_NUM:
@@ -1323,7 +1407,7 @@ bool PWSFilterManager::PassesPWHFiltering(const CItemData *pci) const
           ASSERT(0);
       }
 
-      const int ifunction = (int)st_fldata.rule;
+      const auto ifunction = (int)st_fldata.rule;
       switch (mt) {
         case PWSMatch::MT_STRING:
           for (auto pwshe_iter = pwhistlist.begin(); pwshe_iter != pwhistlist.end(); pwshe_iter++) {
@@ -1450,7 +1534,7 @@ bool PWSFilterManager::PassesPWPFiltering(const CItemData *pci) const
           ASSERT(0);
       }
 
-      const int ifunction = (int)st_fldata.rule;
+      const auto ifunction = (int)st_fldata.rule;
       switch (mt) {
         case PWSMatch::MT_INTEGER:
           thistest_rc = PWSMatch::Match(st_fldata.fnum1, st_fldata.fnum2,
@@ -1528,7 +1612,7 @@ bool PWSFilterManager::PassesAttFiltering(const CItemData *pci, const PWScore &c
           ASSERT(0);
       }
 
-      const int ifunction = (int)st_fldata.rule;
+      const auto ifunction = (int)st_fldata.rule;
       switch (mt) {
         case PWSMatch::MT_BOOL:
           thistest_rc = PWSMatch::Match(bValue, ifunction);
@@ -1583,4 +1667,3 @@ bool PWSFilterManager::PassesAttFiltering(const CItemData *pci, const PWScore &c
   // We finished all the groups and haven't found one that is true - exclude entry.
   return false;
 }
-
