@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -36,6 +36,7 @@ struct BaseEntryParms {
   CItemData::EntryType TargetType;
   int ibasedata;
   bool bMultipleEntriesFound;
+
   BaseEntryParms() : base_uuid(pws_os::CUUID::NullUUID()) {}
 };
 
@@ -119,7 +120,7 @@ public:
   {m_nRecordsWithUnknownFields++;}
 
   // Clear out database structures and associated fields
-  void ClearData();
+  void ClearDBData();
   void ReInit(bool bNewfile = false);
 
   // Following used to read/write databases and Get/Set file name
@@ -143,7 +144,9 @@ public:
                 bool bUpdateSig = true);
   int WriteExportFile(const StringX &filename, OrderedItemList *pOIL,
                       PWScore *pINcore, PWSfile::VERSION version,
-                      std::vector<StringX> &vEmptyGroups, CReport *pRpt = NULL);
+                      std::vector<StringX> &vEmptyGroups, 
+                      bool bExportDBFilters,
+                      std::vector<pws_os::CUUID> &vuuidAddedBases, CReport *pRpt = NULL);
   int WriteV17File(const StringX &filename)
   {return WriteFile(filename, PWSfile::V17, false);}
   int WriteV2File(const StringX &filename)
@@ -242,21 +245,23 @@ public:
   bool IsLockedFile(const stringT &filename) const;
   void UnlockFile(const stringT &filename);
 
+  void SafeUnlockCurFile(); // unlocks current file iff we locked it.
+
   // Following 3 routines only for SaveAs to use a temporary lock handle
   // LockFile2, UnLockFile2 & MoveLock
   bool LockFile2(const stringT &filename, stringT &locker);
   void UnlockFile2(const stringT &filename);
   void MoveLock()
-  {m_lockFileHandle = m_lockFileHandle2; m_lockFileHandle2 = INVALID_HANDLE_VALUE;
-   m_LockCount = m_LockCount2; m_LockCount2 = 0;}
+  {m_lockFileHandle = m_lockFileHandle2; m_lockFileHandle2 = INVALID_HANDLE_VALUE;}
 
   // Set application data
-  void SetApplicationNameAndVersion(const stringT &appName, DWORD dwMajorMinor);
+  void SetApplicationNameAndVersion(const stringT &appName, DWORD dwMajorMinor,
+                                    DWORD dwBuildRevision = 0);
 
   // GetAllGroups - returns an array of all unique group prefix names in m_pwlist
   // e.g., "A", "A.B", "A.B.C"
   // "All" includes empty groups!
-  void GetAllGroups(std::vector<stringT> &vAllGroups) const;
+  void GetAllGroups(std::vector<stringT> &vAllGroups, const bool bIncludeEmptyGroups = true) const;
   // Construct unique title
   StringX GetUniqueTitle(const StringX &group, const StringX &title,
                          const StringX &user, const int IDS_MESSAGE);
@@ -308,9 +313,11 @@ public:
   int Execute(Command *pcmd);
   void Undo();
   void Redo();
-  void ClearCommands();
+  void ClearCommands();  // This should be private to prevent UI calling directly but called by coretest
   bool AnyToUndo() const;
   bool AnyToRedo() const;
+  Command * GetRedoCommand();
+  Command * GetUndoCommand();
 
   // Find in m_pwlist by group, title and user name, exact match
   ItemListIter Find(const StringX &a_group,
@@ -335,6 +342,7 @@ public:
 
   // alias/base and shortcut/base handling
   void SortDependents(UUIDVector &dlist, StringX &csDependents);
+  void SortDependents(UUIDVector &dlist, std::vector<StringX> &vsxDependents);
   size_t NumAliases(const pws_os::CUUID &base_uuid) const
   {return m_base2aliases_mmap.count(base_uuid);}
   size_t NumShortcuts(const pws_os::CUUID &base_uuid) const
@@ -344,18 +352,18 @@ public:
   ItemListIter GetUniqueBase(const StringX &grouptitle, 
                              const StringX &titleuser, bool &bMultiple);
 
-  bool HasDBChanged() const
-  { return m_stDBCS.bDBChanged; }
+  bool HasDBChanged() { return m_DBCurrentState == DIRTY; }
+
   bool HaveDBPrefsChanged() const
-  { return m_stDBCS.bDBPrefsChanged; }
+  { return m_InitialDBPreferences != PWSprefs::GetInstance()->Store(); }
   bool HaveHeaderPreferencesChanged(const StringX &prefString)
   { return _tcscmp(prefString.c_str(), m_hdr.m_prefString.c_str()) != 0; }
   bool HaveEmptyGroupsChanged() const
-  { return m_stDBCS.bEmptyGroupsChanged; }
+  { return m_InitialEmptyGroups != m_vEmptyGroups; }
   bool HavePasswordPolicyNamesChanged() const
-  { return m_stDBCS.bPolicyNamesChanged; }
+  { return m_InitialMapPSWDPLC != m_MapPSWDPLC; }
   bool HaveDBFiltersChanged() const
-  { return m_stDBCS.bDBFiltersChanged; }
+  { return m_InitialMapDBFilters != m_MapDBFilters; }
 
   // Note GroupDisplay & RUE list not checked for Save Immediately processing
   // Also, these are changed by user indirect action and therefore changes are NOT
@@ -363,13 +371,6 @@ public:
   bool HasGroupDisplayChanged() const;
   bool HasRUEListChanged() const;
 
-  // NOTE - GroupDisplay & RUE list are NOT checked via this call
-  bool HasAnythingChanged() const
-  { return m_stDBCS.HasAnythingChanged(); }
-  
-  // PWScore::Execute uses this to set the changed status
-  void GetChangedStatus(Command *pcmd, st_DBChangeStatus &st_Command);
-
   bool ChangeMode(stringT &locker, int &iErrorCode);
   PWSFileSig& GetCurrentFileSig() {return *m_pFileSig;}
 
@@ -382,8 +383,7 @@ public:
   bool GetDBNotificationState()
   {return m_bNotifyDB;}
 
-  void GUISetupDisplayInfo(CItemData &ci);
-  void GUIRefreshEntry(const CItemData &ci);
+  void GUIRefreshEntry(const CItemData &ci, bool bAllowFail = false);
   void UpdateWizard(const stringT &s);
 
   // Get/Set Display information from/to database
@@ -425,6 +425,7 @@ public:
 
   // Empty Groups
   const std::vector<StringX> & GetEmptyGroups() const {return m_vEmptyGroups;}
+  const std::vector<StringX> & GetSavedEmptyGroups() const { return m_InitialEmptyGroups; }
   bool IsEmptyGroup(const StringX &sxEmptyGroup) const;
   size_t GetNumberEmptyGroups() const {return m_vEmptyGroups.size();}
 
@@ -448,6 +449,7 @@ protected:
   bool m_isAuxCore; // set in c'tor, if true, never update prefs from DB.  
 
 private:
+
   // Database update routines
 
   // NOTE: Member functions starting with 'Do' or 'Undo' are meant to
@@ -481,8 +483,9 @@ private:
                                       SavePWHistoryMap &mapSavedHistory);
   virtual void UndoUpdatePasswordHistory(SavePWHistoryMap &mapSavedHistory);
 
-  virtual int DoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath);
-  virtual void UndoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath);
+  virtual int DoRenameGroup(const StringX &sxOldPath, const StringX &sxNewPath,
+                            MultiCommands * &pmulticmds);
+  virtual void UndoRenameGroup(MultiCommands *pmulticmds);
 
   virtual int DoChangeHeader(const StringX &sxNewValue, const PWSfile::HeaderType ht);
   virtual void UndoChangeHeader(const StringX &sxOldValue, const PWSfile::HeaderType ht);
@@ -497,16 +500,11 @@ private:
   //***** Make all calls that change the core private
   //   This excludes Group Display and RUE List which should not be via 
   //   Commands as no requirement to Undo/Redo and whose save is UI driven.
-  void SetChangedStatus(); // Used by Execute/Undo/Redo
   void SetInitialValues(); // Called after successful read/write of a database
 
   // Update header
   int SetHeaderItem(const StringX &sxNewValue, PWSfile::HeaderType ht);
 
-  // Update DB preferences - not sure why this is still here (called by DBPrefsCommand::Execute)
-  void SetDBPrefsChanged(bool bDBprefschanged)
-  { m_stDBCS.bDBPrefsChanged = bDBprefschanged; }
-
   // Set empty groups
   bool SetEmptyGroups(const std::vector<StringX> &vEmptyGroups);
 
@@ -557,8 +555,6 @@ private:
 
   HANDLE m_lockFileHandle;
   HANDLE m_lockFileHandle2;
-  int m_LockCount;
-  int m_LockCount2;
 
   stringT m_AppNameAndVersion;
   PWSfile::VERSION m_ReadFileVersion;
@@ -568,9 +564,7 @@ private:
   bool m_bNotifyDB;
   bool m_bIsOpen;
 
-  st_DBChangeStatus m_stDBCS;
-
-  PWSfileHeader m_hdr;
+    PWSfileHeader m_hdr;
   StringX m_InitialDBName, m_InitialDBDesc;
   StringX m_InitialDBPreferences;
   std::vector<bool> m_InitialDisplayStatus; // for HasGroupDisplayChanged (stored in header)
@@ -594,16 +588,12 @@ private:
   const ItemMMap &GetBase2ShortcutsMmap() const {return m_base2shortcuts_mmap;}
   void SetBase2ShortcutsMmap(ItemMMap &b2smm) {m_base2shortcuts_mmap = b2smm;}
   
-  // Changed groups
-  std::vector<StringX> m_vNodes_Modified;
-  std::vector<StringX> m_saved_vNodes_Modified;
-
   // Following are private in PWScore, public in CommandInterface:
   void AddChangedNodes(StringX path);
 
   // EmptyGroups
   std::vector<StringX> m_vEmptyGroups;
-  std::vector<StringX> m_InitialvEmptyGroups;
+  std::vector<StringX> m_InitialEmptyGroups;
 
   UnknownFieldList m_UHFL;
   int m_nRecordsWithUnknownFields;
@@ -615,9 +605,12 @@ private:
   std::bitset<UIInterFace::NUM_SUPPORTED> m_bsSupportedFunctions;
   
   void NotifyGUINeedsUpdating(UpdateGUICommand::GUI_Action, const pws_os::CUUID &,
-                              CItemData::FieldType ft = CItemData::START,
-                              bool bUpdateGUI = true);
+                              CItemData::FieldType ft = CItemData::START);
 
+  // Version for groups
+  void NotifyGUINeedsUpdating(UpdateGUICommand::GUI_Action ga,
+                              const std::vector<StringX> &vGroups);
+  
   // Create header for included(Text) and excluded(XML) exports
   StringX BuildHeader(const CItemData::FieldBits &bsFields, const bool bIncluded);
 
@@ -625,6 +618,18 @@ private:
   std::vector<Command *> m_vpcommands;
   std::vector<Command *>::iterator m_undo_iter;
   std::vector<Command *>::iterator m_redo_iter;
+  
+  // DB clean/dirty states - before and after command execution.
+  enum DBState { CLEAN, DIRTY };
+  struct DBStates {
+    DBState before;
+    DBState after;
+  };
+
+  std::vector<DBStates> m_vDBState;
+  std::vector<DBStates>::iterator m_undo_DBState_iter;
+  std::vector<DBStates>::iterator m_redo_DBState_iter;
+  DBState m_DBCurrentState;
 
   static Reporter *m_pReporter; // set as soon as possible to show errors
   static Asker *m_pAsker;
