@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2013-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -76,7 +76,7 @@ PWSfileV4::CKeyBlocks::~CKeyBlocks()
 {
 }
 
-PWSfileV4::CKeyBlocks PWSfileV4::CKeyBlocks::operator=(const PWSfileV4::CKeyBlocks &that)
+PWSfileV4::CKeyBlocks& PWSfileV4::CKeyBlocks::operator=(const PWSfileV4::CKeyBlocks &that)
 {
   if (this != &that) {
     m_kbs = that.m_kbs;
@@ -84,7 +84,6 @@ PWSfileV4::CKeyBlocks PWSfileV4::CKeyBlocks::operator=(const PWSfileV4::CKeyBloc
   return *this;
 }
 
-
 PWSfileV4::PWSfileV4(const StringX &filename, RWmode mode, VERSION version)
   : PWSfile(filename, mode, version),
     m_effectiveFileLength(0), m_nHashIters(MIN_HASH_ITERATIONS)
@@ -211,7 +210,6 @@ int PWSfileV4::CheckPasskey(const StringX &filename,
   PWS_LOGIT;
 
   FILE *fd = a_fd;
-  int retval = SUCCESS;
   SHA256 H;
 
   if (fd == NULL) {
@@ -220,7 +218,7 @@ int PWSfileV4::CheckPasskey(const StringX &filename,
   if (fd == NULL)
     return CANT_OPEN_FILE;
 
-  retval = SanityCheck(fd);
+  int retval = SanityCheck(fd);
   if (retval == SUCCESS) {
     PWSfileV4 pv4(filename, Read, V40);
     pv4.m_fd = fd;
@@ -252,7 +250,7 @@ size_t PWSfileV4::WriteCBC(unsigned char type, const unsigned char *data,
 
   m_hmac.Update(&type, 1);
   m_hmac.Update(buf, sizeof(buf));
-  m_hmac.Update(data, length);
+  m_hmac.Update(data, (unsigned long)length);
 
   return PWSfile::WriteCBC(type, data, length);
 }
@@ -311,7 +309,7 @@ size_t PWSfileV4::WriteContentFields(unsigned char *content, size_t len)
   _writecbc(m_fd, content, len, &fish, IV);
 
   // update content's HMAC
-  hmac.Update(content, len);
+  hmac.Update(content, (unsigned long)len);
 
   // write content's HMAC
   unsigned char digest[SHA256::HASHLEN];
@@ -345,7 +343,7 @@ size_t PWSfileV4::ReadCBC(unsigned char &type, unsigned char* &data,
 
     m_hmac.Update(&type, 1);
     m_hmac.Update(buf, sizeof(buf));
-    m_hmac.Update(data, length);
+    m_hmac.Update(data, (unsigned long)length);
   }
 
   return numRead;
@@ -391,7 +389,7 @@ int PWSfileV4::ReadRecord(CItemAtt &att)
 {
   ASSERT(m_fd != NULL);
   ASSERT(m_curversion == V40);
-  if (unsigned(ftell(m_fd)) < m_effectiveFileLength)
+  if (unsigned(GetOffset()) < m_effectiveFileLength)
     return att.Read(this);
   else
     return END_OF_FILE;
@@ -412,7 +410,7 @@ void PWSfileV4::StretchKey(const unsigned char *salt, unsigned long saltLen,
 
   HMAC<SHA256, SHA256::HASHLEN, SHA256::BLOCKSIZE> hmac;
   ConvertString(passkey, pstr, passLen);
-  pbkdf2(pstr, passLen, salt, saltLen, N, &hmac, Ptag, &PtagLen);
+  pbkdf2(pstr, (unsigned long)passLen, salt, saltLen, N, &hmac, Ptag, &PtagLen);
 
 #ifdef UNICODE
   trashMemory(pstr, passLen);
@@ -493,7 +491,6 @@ void PWSfileV4::ComputeEndKB(const unsigned char hnonce[SHA256::HASHLEN],
   m_hmac.Final(digest);
 }
 
-
 #define SAFE_FWRITE(p, sz, cnt, stream) \
   { \
     size_t _ret = fwrite(p, sz, cnt, stream); \
@@ -628,8 +625,8 @@ int PWSfileV4::WriteHeader()
   // Write out who saved it!
   {
     const SysInfo *si = SysInfo::GetInstance();
-    stringT user = si->GetRealUser();
-    stringT sysname = si->GetRealHost();
+    const stringT &user = si->GetRealUser();
+    const stringT &sysname = si->GetRealHost();
     numWritten = WriteCBC(HDR_LASTUPDATEUSER, user.c_str());
     if (numWritten > 0)
       numWritten = WriteCBC(HDR_LASTUPDATEHOST, sysname.c_str());
@@ -666,11 +663,11 @@ int PWSfileV4::WriteHeader()
       num = 255;  // Only save up to max as defined by FormatV3.
 
     size_t buflen = (num * sizeof(uuid_array_t)) + 1;
-    unsigned char *buf = new unsigned char[buflen];
+    auto *buf = new unsigned char[buflen];
     buf[0] = (unsigned char)num;
     unsigned char *buf_ptr = buf + 1;
 
-    UUIDListIter iter = m_hdr.m_RUEList.begin();
+    auto iter = m_hdr.m_RUEList.begin();
     
     for (size_t n = 0; n < num; n++, iter++) {
       const uuid_array_t *rep = iter->GetARep();
@@ -708,7 +705,7 @@ int PWSfileV4::WriteHeader()
     }
 
     // Allocate buffer in calculated size
-    unsigned char *buf = new unsigned char[totlen];
+    auto *buf = new unsigned char[totlen];
     memset(buf, 0, totlen); // in case we truncate some names, don't leak info.
 
     // fill buffer
@@ -737,10 +734,10 @@ int PWSfileV4::WriteHeader()
                                           pwpol.symbols.c_str(), pwpol.symbols.length());
       if (symSetLen > 255) // too bad if too long...
         symSetLen = 255;
-        *buf_ptr++ = (unsigned char)symSetLen;
-        pws_os::wcstombs((char *)buf_ptr, symSetLen,
-                         pwpol.symbols.c_str(), pwpol.symbols.length());
-        buf_ptr += symSetLen;
+      *buf_ptr++ = (unsigned char)symSetLen;
+      pws_os::wcstombs((char *)buf_ptr, symSetLen,
+                       pwpol.symbols.c_str(), pwpol.symbols.length());
+      buf_ptr += symSetLen;
       }
     } // for loop over policies
 
@@ -838,10 +835,10 @@ bool PWSfileV4::VerifyKeyBlocks()
   unsigned char ReadEndKB[SHA256::HASHLEN];
   unsigned char CalcEndKB[SHA256::HASHLEN];
 
-  int nRead = fread(hnonce, sizeof(hnonce), 1, m_fd);
+  int nRead = (int)fread(hnonce, sizeof(hnonce), 1, m_fd);
   if (nRead != 1)
     return false;
-  nRead = fread(ReadEndKB, sizeof(ReadEndKB), 1, m_fd);
+  nRead = (int)fread(ReadEndKB, sizeof(ReadEndKB), 1, m_fd);
   if (nRead != 1)
     return false;
 
@@ -962,7 +959,7 @@ bool PWSfileV4::CKeyBlocks::RemoveKeyBlock(const StringX &passkey)
     return false;
 
   KeyBlockFinder find_kb(passkey);
-  const unsigned long old_size = m_kbs.size();
+  const auto old_size = m_kbs.size();
   m_kbs.erase(remove_if(m_kbs.begin(), m_kbs.end(), find_kb),
                m_kbs.end());
 
@@ -1190,7 +1187,7 @@ int PWSfileV4::ReadHeader()
 
           int nameLen = *buf_ptr++;
           // need to tack on null byte to name before conversion
-          unsigned char *nmbuf = new unsigned char[nameLen + 1];
+          auto *nmbuf = new unsigned char[nameLen + 1];
           memcpy(nmbuf, buf_ptr, nameLen); nmbuf[nameLen] = 0;
           utf8status = m_utf8conv.FromUTF8(nmbuf, nameLen, sxPolicyName);
           trashMemory(nmbuf, nameLen); delete[] nmbuf;
@@ -1210,7 +1207,7 @@ int PWSfileV4::ReadHeader()
           int symLen = *buf_ptr++;
           if (symLen > 0) {
             // need to tack on null byte to symbols before conversion
-            unsigned char *symbuf = new unsigned char[symLen + 1];
+            auto *symbuf = new unsigned char[symLen + 1];
             memcpy(symbuf, buf_ptr, symLen); symbuf[symLen] = 0;
             utf8status = m_utf8conv.FromUTF8(symbuf, symLen, pwp.symbols);
             trashMemory(symbuf, symLen); delete[] symbuf;
@@ -1222,7 +1219,7 @@ int PWSfileV4::ReadHeader()
             break; // Error
           pair< map<StringX, PWPolicy>::iterator, bool > pr;
           pr = m_MapPSWDPLC.insert(PSWDPolicyMapPair(sxPolicyName, pwp));
-          if (pr.second == false) break; // Error
+          if (!pr.second) break; // Error
         } // iterate over named policies
       }
       break;
@@ -1259,6 +1256,9 @@ int PWSfileV4::ReadHeader()
     delete[] utf8; utf8 = NULL; utf8Len = 0;
   } while (fieldType != HDR_END);
 
+  // Now sort it for when we compare.
+  std::sort(m_vEmptyGroups.begin(), m_vEmptyGroups.end());
+
   return SUCCESS;
 }
 
