@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -125,7 +125,8 @@ bool CRUEList::GetPWEntry(size_t index, CItemData &ci) const
     // Entry does not exist anymore!
     // We break constness here since this is "housekeeping", and
     // doesn't affect the object's semantics
-    const_cast<CRUEList *>(this)->m_RUEList.erase(m_RUEList.begin() + index);
+    RUEList::iterator iter = const_cast<CRUEList *>(this)->m_RUEList.begin() + index;
+    const_cast<CRUEList *>(this)->m_RUEList.erase(iter);
     return false;
   } else { // valid pw_listpos
     ci = m_core.GetEntry(pw_listpos);
@@ -150,14 +151,5 @@ void CRUEList::SetRUEList(const UUIDList &RUElist)
     m_RUEList.resize(m_maxentries);
 }
 
-CRUEList& CRUEList::operator=(const CRUEList &that)
-{
-  if (this != &that) {
-    m_maxentries = that.m_maxentries;
-    m_RUEList = that.m_RUEList;
-  }
-  return *this;
-}
-
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
