@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -13,6 +13,7 @@
 
 #include "sha256.h"
 #include "StringX.h"
+#include "StringXStream.h"
 #include "Fish.h"
 #include "PwsPlatform.h"
 #include "UTF8Conv.h"
@@ -53,7 +54,7 @@ extern size_t _readcbc(FILE *fp, unsigned char * &buffer,
                        size_t &buffer_len,
                        unsigned char &type, Fish *Algorithm,
                        unsigned char *cbcbuffer,
-                       const unsigned char *TERMINAL_BLOCK = NULL, 
+                       const unsigned char *TERMINAL_BLOCK = NULL,
                        ulong64 file_len = 0);
 
 // typeless version for V4 content (caller pre-allocates buffer)
@@ -70,7 +71,6 @@ extern size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length,
 extern size_t _writecbc(FILE *fp, const unsigned char *buffer, size_t length,
                         Fish *Algorithm, unsigned char *cbcbuffer);
 
-
 // The following can be used directly or via template functions getInt<> / putInt<>
 
 /*
@@ -228,7 +228,6 @@ T inline getInt(const unsigned char *buf) { return GetPutImpl<T, sizeof(T)>::Get
 template<typename T>
 void inline putInt(unsigned char *buf, const T val) { GetPutImpl<T, sizeof(T)>::PutInt(buf, val); }
 
-
 bool operator==(const std::string& str1, const stringT& str2);
 inline bool operator==(const stringT& str1, const std::string &str2) { return str2 == str1; }
 
@@ -260,18 +259,20 @@ namespace PWSUtil {
   stringT GetSafeXMLString(const StringX &sxInString);
 
   bool pull_time(time_t &t, const unsigned char *data, size_t len);
+  // load file to stream
+  bool loadFile(const StringX &filename, StringXStream &stream);
 }
 
 ///////////////////////////////////////////////////////
 // Following two templates lets us use the two types
-// of iterators in a common (templatized) function when 
+// of iterators in a common (templatized) function when
 // all we need to do is to access the underlying value
 template <typename PairAssociativeContainer>
 class get_second {
   public:
     typedef typename PairAssociativeContainer::mapped_type mapped_type;
     typedef typename PairAssociativeContainer::const_iterator const_iterator;
-    const mapped_type& operator()(const_iterator val) { return val->second; }
+    const mapped_type& operator()(const_iterator val) const { return val->second; }
 };
 
 template <typename SequenceContainer>
@@ -279,10 +280,16 @@ class dereference {
   public:
     typedef typename SequenceContainer::value_type value_type;
     typedef typename SequenceContainer::const_iterator const_iterator;
-    const value_type& operator()(const_iterator itr) { return *itr; }
+    const value_type& operator()(const_iterator itr) const { return *itr; }
 };
 
 extern unsigned int GetStringBufSize(const TCHAR *fmt, va_list args);
+
+bool FindNoCase( const StringX& src, const StringX& dest);
+
+
+std::string toutf8(const std::wstring &w);
+
 #endif /* __UTIL_H */
 //-----------------------------------------------------------------------------
 // Local variables:
