@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -604,19 +604,6 @@ void XMLFileHandlers::AddXMLEntries()
     m_pmulticmds->Add(pcmd);
   }
 
-  // Then add any Empty Groups imported that are not already in the database
-  if (!m_vEmptyGroups.empty()) {
-    if (!m_ImportedPrefix.empty()) {
-      const StringX sxNewPath = StringX(m_ImportedPrefix.c_str()) + StringX(L".");
-      for (size_t i = 0; i < m_vEmptyGroups.size(); i++) {
-        m_vEmptyGroups[i] = sxNewPath + m_vEmptyGroups[i];
-      }
-    }
-    Command *pcmd = DBEmptyGroupsCommand::Create(m_pXMLcore, m_vEmptyGroups,
-                           DBEmptyGroupsCommand::EG_ADDALL);
-    m_pmulticmds->Add(pcmd);
-  }
-
   // Get current DB default password policy and that from the XML file and
   // check that they are the same?
   PWPolicy st_to_default_pp, st_import_default_pp;
@@ -964,6 +951,7 @@ void XMLFileHandlers::AddXMLEntries()
         m_prpt->WriteLine(sxTemp.c_str());
         m_numShortcutsRemoved++;
       }
+
       // Check if already in use as an the PaswordSafe Application HotKey
       if (m_pXMLcore->GetAppHotKey() == iKBShortcut) {
         // Remove it
@@ -977,7 +965,7 @@ void XMLFileHandlers::AddXMLEntries()
         m_numShortcutsRemoved++;
       }
     }
-    m_pXMLcore->GUISetupDisplayInfo(ci_temp);
+
     Command *pcmd = AddEntryCommand::Create(m_pXMLcore, ci_temp);
     pcmd->SetNoGUINotify();
     m_pmulticmds->Add(pcmd);
@@ -995,6 +983,40 @@ void XMLFileHandlers::AddXMLEntries()
   pcmdS->SetNoGUINotify();
   m_pmulticmds->Add(pcmdS);
 
+  // Validate Empty Groups don't have empty sub-groups
+  if (!m_vEmptyGroups.empty()) {
+    std::sort(m_vEmptyGroups.begin(), m_vEmptyGroups.end());
+    std::vector<size_t> viDelete;
+    for (size_t ieg = 0; ieg < m_vEmptyGroups.size() - 1; ieg++) {
+      StringX sxEG = m_vEmptyGroups[ieg] + L".";
+      if (sxEG == m_vEmptyGroups[ieg + 1].substr(0, sxEG.length())) {
+        // Can't be empty as has empty sub-group. Save to delete later
+        viDelete.push_back(ieg);
+      }
+    }
+
+    if (!viDelete.empty()) {
+      // Remove non-empty groups
+      std::vector<size_t>::reverse_iterator rit;
+      for (rit = viDelete.rbegin(); rit != viDelete.rend(); rit++) {
+        m_vEmptyGroups.erase(m_vEmptyGroups.begin() + *rit);
+      }
+    }
+  }
+  
+  // Then add any Empty Groups imported that are not already in the database
+  if (!m_vEmptyGroups.empty()) {
+    if (!m_ImportedPrefix.empty()) {
+      const StringX sxNewPath = StringX(m_ImportedPrefix.c_str()) + StringX(L".");
+      for (size_t i = 0; i < m_vEmptyGroups.size(); i++) {
+        m_vEmptyGroups[i] = sxNewPath + m_vEmptyGroups[i];
+      }
+    }
+    Command *pcmd = DBEmptyGroupsCommand::Create(m_pXMLcore, m_vEmptyGroups,
+                           DBEmptyGroupsCommand::EG_ADDALL);
+    m_pmulticmds->Add(pcmd);
+  }
+
   Command *pcmd2 = UpdateGUICommand::Create(m_pXMLcore,
                                             UpdateGUICommand::WN_EXECUTE_REDO,
                                             UpdateGUICommand::GUI_REDO_IMPORT);
