@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -20,7 +20,12 @@ namespace pws_os {
    *
    * Linux: 'type' maps to 'flags' for dlopen()
    */
-   enum loadLibraryTypes { LOAD_LIBRARY_SYS, LOAD_LIBRARY_APP, LOAD_LIBRARY_CUSTOM };
+   enum loadLibraryTypes { 
+	   LOAD_LIBRARY_SYS, 
+	   LOAD_LIBRARY_APP, 
+	   LOAD_LIBRARY_CUSTOM,
+	   LOAD_LIBRARY_RESOURCE
+   };
    extern void *LoadLibrary(const TCHAR *lib, int type);
    extern void *GetFunction(void *handle, const char *name);
    extern bool FreeLibrary(void *handle);
