@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -34,14 +34,12 @@
 #include "graphics/toolbar/new/copypassword_disabled.xpm"
 ////@end XPM images
 
-
 /*!
  * CManagePasswordPolicies type definition
  */
 
 IMPLEMENT_CLASS( CManagePasswordPolicies, wxDialog )
 
-
 /*!
  * CManagePasswordPolicies event table definition
  */
@@ -77,12 +75,10 @@ BEGIN_EVENT_TABLE( CManagePasswordPolicies, wxDialog )
 
 END_EVENT_TABLE()
 
-
 /*!
  * CManagePasswordPolicies constructor
  */
 
-
 CManagePasswordPolicies::CManagePasswordPolicies( wxWindow* parent,  PWScore &core, wxWindowID id,
               const wxString& caption, const wxPoint& pos,
               const wxSize& size, long style )
@@ -94,7 +90,6 @@ CManagePasswordPolicies::CManagePasswordPolicies( wxWindow* parent,  PWScore &co
   Create(parent, id, caption, pos, size, style);
 }
 
-
 /*!
  * CManagePasswordPolicies creator
  */
@@ -115,7 +110,6 @@ bool CManagePasswordPolicies::Create( wxWindow* parent, wxWindowID id, const wxS
   return true;
 }
 
-
 /*!
  * CManagePasswordPolicies destructor
  */
@@ -126,7 +120,6 @@ CManagePasswordPolicies::~CManagePasswordPolicies()
 ////@end CManagePasswordPolicies destruction
 }
 
-
 /*!
  * Member initialisation
  */
@@ -145,7 +138,6 @@ void CManagePasswordPolicies::Init()
   m_st_default_pp = PWSprefs::GetInstance()->GetDefaultPolicy();
 }
 
-
 /*!
  * Control creation for CManagePasswordPolicies
  */
@@ -155,14 +147,14 @@ void CManagePasswordPolicies::CreateControls()
 ////@begin CManagePasswordPolicies content construction
   CManagePasswordPolicies* itemDialog1 = this;
 
-  wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
+  auto *itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
   itemDialog1->SetSizer(itemBoxSizer2);
 
   wxStaticText* itemStaticText3 = new wxStaticText( itemDialog1, wxID_STATIC, _("Available Password Policies:"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer2->Add(itemStaticText3, 0, wxALIGN_LEFT|wxALL, 5);
 
-  wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
-  itemBoxSizer2->Add(itemBoxSizer4, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
+  auto *itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
+  itemBoxSizer2->Add(itemBoxSizer4, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
 
   m_PolicyNames = new wxGrid( itemDialog1, ID_POLICYLIST, wxDefaultPosition, wxSize(269, 150), wxSUNKEN_BORDER|wxHSCROLL|wxVSCROLL );
   m_PolicyNames->SetDefaultColSize(100);
@@ -170,12 +162,12 @@ void CManagePasswordPolicies::CreateControls()
   m_PolicyNames->SetColLabelSize(25);
   m_PolicyNames->SetRowLabelSize(50);
   m_PolicyNames->CreateGrid(10, 2, wxGrid::wxGridSelectRows);
-  itemBoxSizer4->Add(m_PolicyNames, 3, wxGROW|wxALL, 5);
+  itemBoxSizer4->Add(m_PolicyNames, 3, wxEXPAND|wxALL, 5);
 
-  wxBoxSizer* itemBoxSizer6 = new wxBoxSizer(wxVERTICAL);
+  auto *itemBoxSizer6 = new wxBoxSizer(wxVERTICAL);
   itemBoxSizer4->Add(itemBoxSizer6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  wxGridSizer* itemGridSizer7 = new wxGridSizer(0, 2, 0, 0);
+  auto *itemGridSizer7 = new wxGridSizer(0, 2, 0, 0);
   itemBoxSizer6->Add(itemGridSizer7, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
 
   wxButton* itemButton8 = new wxButton( itemDialog1, wxID_NEW, _("&New"), wxDefaultPosition, wxDefaultSize, 0 );
@@ -197,13 +189,13 @@ void CManagePasswordPolicies::CreateControls()
   itemGridSizer7->Add(itemButton13, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxStaticBox* itemStaticBoxSizer14Static = new wxStaticBox(itemDialog1, wxID_STATIC, _("Test Selected Policy"));
-  wxStaticBoxSizer* itemStaticBoxSizer14 = new wxStaticBoxSizer(itemStaticBoxSizer14Static, wxVERTICAL);
+  auto *itemStaticBoxSizer14 = new wxStaticBoxSizer(itemStaticBoxSizer14Static, wxVERTICAL);
   itemBoxSizer6->Add(itemStaticBoxSizer14, 0, wxGROW|wxALL, 5);
 
   wxButton* itemButton15 = new wxButton( itemDialog1, ID_GENERATE_PASSWORD, _("Generate"), wxDefaultPosition, wxDefaultSize, 0 );
   itemStaticBoxSizer14->Add(itemButton15, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
 
-  wxBoxSizer* itemBoxSizer16 = new wxBoxSizer(wxHORIZONTAL);
+  auto *itemBoxSizer16 = new wxBoxSizer(wxHORIZONTAL);
   itemStaticBoxSizer14->Add(itemBoxSizer16, 0, wxGROW|wxALL, 5);
 
   m_passwordCtrl = new wxTextCtrl( itemDialog1, ID_PASSWORD_TXT, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
@@ -219,16 +211,16 @@ void CManagePasswordPolicies::CreateControls()
   m_lowerTableDesc = new wxStaticText( itemDialog1, wxID_STATIC, _("Selected policy details:"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer2->Add(m_lowerTableDesc, 0, wxALIGN_LEFT|wxALL, 5);
 
-  wxBoxSizer* itemBoxSizer20 = new wxBoxSizer(wxHORIZONTAL);
-  itemBoxSizer2->Add(itemBoxSizer20, 0, wxGROW|wxALL, 5);
+  auto *itemBoxSizer20 = new wxBoxSizer(wxHORIZONTAL);
+  itemBoxSizer2->Add(itemBoxSizer20, 1, wxEXPAND|wxALL, 5);
 
   m_PolicyDetails = new wxGrid( itemDialog1, ID_POLICYPROPERTIES, wxDefaultPosition, wxSize(-1, 150), wxSUNKEN_BORDER|wxHSCROLL|wxVSCROLL );
-  m_PolicyDetails->SetDefaultColSize(210);
+  m_PolicyDetails->SetDefaultColSize(220);
   m_PolicyDetails->SetDefaultRowSize(25);
   m_PolicyDetails->SetColLabelSize(25);
   m_PolicyDetails->SetRowLabelSize(50);
   m_PolicyDetails->CreateGrid(5, 2, wxGrid::wxGridSelectRows);
-  itemBoxSizer20->Add(m_PolicyDetails, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer20->Add(m_PolicyDetails, 1, wxEXPAND|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   m_PolicyEntries = new wxGrid( itemDialog1, ID_POLICYENTRIES, wxDefaultPosition, wxSize(-1, 150), wxSUNKEN_BORDER|wxHSCROLL|wxVSCROLL );
   m_PolicyEntries->SetDefaultColSize(140);
@@ -236,9 +228,9 @@ void CManagePasswordPolicies::CreateControls()
   m_PolicyEntries->SetColLabelSize(25);
   m_PolicyEntries->SetRowLabelSize(50);
   m_PolicyEntries->CreateGrid(5, 3, wxGrid::wxGridSelectRows);
-  itemBoxSizer20->Add(m_PolicyEntries, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  itemBoxSizer20->Add(m_PolicyEntries, 1, wxEXPAND|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  wxStdDialogButtonSizer* itemStdDialogButtonSizer23 = new wxStdDialogButtonSizer;
+  auto *itemStdDialogButtonSizer23 = new wxStdDialogButtonSizer;
 
   itemBoxSizer2->Add(itemStdDialogButtonSizer23, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
   wxButton* itemButton24 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
@@ -305,7 +297,6 @@ void CManagePasswordPolicies::CreateControls()
   m_PolicyNames->SetFocus();
 }
 
-
 /*!
  * Should we show tooltips?
  */
@@ -358,7 +349,6 @@ bool CManagePasswordPolicies::Show(bool show)
   return wxDialog::Show(show);
 }
 
-
 void CManagePasswordPolicies::ShowPolicyDetails()
 {
   m_bViewPolicy = true;
@@ -413,7 +403,7 @@ static void wxRowPutter(int row, const stringT &name, const stringT &value,
                              void *table)
 {
   // Callback function used by PWPolicy::Policy2Table
-  wxGrid *tableControl = (wxGrid *)table;
+  auto *tableControl = (wxGrid *)table;
   tableControl->InsertRows(row);
   tableControl->SetCellValue(row, 0, name.c_str());
   tableControl->SetCellValue(row, 1, value.c_str());
@@ -441,7 +431,7 @@ PWPolicy CManagePasswordPolicies::GetSelectedPolicy() const
   if (row > 0) {
     const wxString policyname = m_PolicyNames->GetCellValue(row, 0);
 
-    PSWDPolicyMapCIter iter = m_MapPSWDPLC.find(tostringx(policyname));
+    auto iter = m_MapPSWDPLC.find(tostringx(policyname));
     if (iter == m_MapPSWDPLC.end())
       return m_st_default_pp;
 
@@ -542,7 +532,6 @@ void CManagePasswordPolicies::OnNewClick( wxCommandEvent& )
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_EDIT
  */
@@ -558,8 +547,8 @@ void CManagePasswordPolicies::OnEditPpClick( wxCommandEvent& )
   if (row == 0) { // 1st row is default
     st_pp = m_st_default_pp;
   } else {
-    PSWDPolicyMapIter mapIter = m_MapPSWDPLC.find(StringX(policyname.c_str()));
-    if (row != 0 && mapIter == m_MapPSWDPLC.end()) {
+    auto mapIter = m_MapPSWDPLC.find(StringX(policyname.c_str()));
+    if (mapIter == m_MapPSWDPLC.end()) {
       ASSERT(0);
       return;
     }
@@ -576,7 +565,6 @@ void CManagePasswordPolicies::OnEditPpClick( wxCommandEvent& )
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DELETE
  */
@@ -589,7 +577,6 @@ void CManagePasswordPolicies::OnDeleteClick( wxCommandEvent& event )
 ////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DELETE in CManagePasswordPolicies.
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_LIST
  */
@@ -602,7 +589,6 @@ void CManagePasswordPolicies::OnListClick( wxCommandEvent&  )
     ShowPolicyDetails();
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_UNDO
  */
@@ -615,7 +601,6 @@ void CManagePasswordPolicies::OnUndoClick( wxCommandEvent& event )
 ////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_UNDO in CManagePasswordPolicies.
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_REDO
  */
@@ -628,7 +613,6 @@ void CManagePasswordPolicies::OnRedoClick( wxCommandEvent& event )
 ////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_REDO in CManagePasswordPolicies.
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
  */
@@ -670,7 +654,6 @@ void CManagePasswordPolicies::OnOkClick( wxCommandEvent& )
   EndModal(wxID_OK);
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL
  */
@@ -683,7 +666,6 @@ void CManagePasswordPolicies::OnCancelClick( wxCommandEvent& event )
 ////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_CANCEL in CManagePasswordPolicies.
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
  */
@@ -696,7 +678,6 @@ void CManagePasswordPolicies::OnHelpClick( wxCommandEvent& event )
 ////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP in CManagePasswordPolicies.
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GENERATE_PASSWORD
  */
@@ -710,7 +691,6 @@ void CManagePasswordPolicies::OnGeneratePasswordClick( wxCommandEvent& event )
   m_passwordCtrl->SetValue(passwd.c_str());
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BITMAPBUTTON
  */
@@ -720,7 +700,6 @@ void CManagePasswordPolicies::OnCopyPasswordClick( wxCommandEvent& )
   PWSclipboard::GetInstance()->SetData(tostringx(m_passwordCtrl->GetValue()));
 }
 
-
 /*!
  * wxEVT_GRID_SELECT_CELL event handler for ID_POLICYLIST
  */
@@ -732,4 +711,3 @@ void CManagePasswordPolicies::OnSelectCell( wxGridEvent& evt )
     UpdateDetails();
   }
 }
-
