@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -46,7 +46,6 @@ using pws_os::CUUID;
 
 IMPLEMENT_CLASS( PWSGrid, wxGrid )
 
-
 /*!
  * PWSGrid event table definition
  */
@@ -81,7 +80,6 @@ PWSGrid::PWSGrid(wxWindow* parent, PWScore &core,
   Create(parent, id, pos, size, style);
 }
 
-
 /*!
  * PWSGrid creator
  */
@@ -102,7 +100,6 @@ bool PWSGrid::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos, const
   return true;
 }
 
-
 /*!
  * PWSGrid destructor
  */
@@ -113,7 +110,6 @@ PWSGrid::~PWSGrid()
 ////@end PWSGrid destruction
 }
 
-
 /*!
  * Member initialisation
  */
@@ -124,7 +120,6 @@ void PWSGrid::Init()
 ////@end PWSGrid member initialisation
 }
 
-
 /*!
  * Control creation for PWSGrid
  */
@@ -208,7 +203,7 @@ void PWSGrid::UpdateItem(const CItemData &item)
 {
   uuid_array_t uuid;
   item.GetUUID(uuid);
-  UUIDRowMapT::iterator iter = m_uuid_map.find(CUUID(uuid));
+  auto iter = m_uuid_map.find(CUUID(uuid));
   if (iter != m_uuid_map.end()) {
     int row = iter->second;
     DeleteRows(row);
@@ -252,9 +247,8 @@ struct moveup : public std::binary_function<UUIDRowMapT::value_type, int, void>
 
 void PWSGrid::Remove(const CUUID &uuid)
 {
-  UUIDRowMapT::iterator iter = m_uuid_map.find(uuid);
+  auto iter = m_uuid_map.find(uuid);
   if (iter != m_uuid_map.end()) {
-
     const int row = iter->second;
 
     //The UI element must be removed first, since the entry in m_core is deleted after
@@ -315,15 +309,15 @@ size_t PWSGrid::GetNumItems() const
 void PWSGrid::DeleteItems(int row, size_t numItems)
 {
   for (size_t N = 0; N < numItems; ++N) {
-    RowUUIDMapT::iterator iter = m_row_map.find(row);
+    auto iter = m_row_map.find(row);
     if (iter != m_row_map.end()) {
-      UUIDRowMapT::iterator iter_uuid = m_uuid_map.find(iter->second);
+      auto iter_uuid = m_uuid_map.find(iter->second);
       m_row_map.erase(iter);
       if (iter_uuid != m_uuid_map.end()) {
         uuid_array_t uuid;
         iter_uuid->first.GetARep(uuid);
         m_uuid_map.erase(iter_uuid);
-        ItemListIter citer = m_core.Find(uuid);
+        auto citer = m_core.Find(uuid);
         if (citer != m_core.GetEntryEndIter()){
           m_core.SuspendOnDBNotification();
           m_core.Execute(DeleteEntryCommand::Create(&m_core,
@@ -393,11 +387,11 @@ CItemData *PWSGrid::GetItem(int row) const
 {
   if (row < 0 || row > const_cast<PWSGrid *>(this)->GetNumberRows())
     return NULL;
-  RowUUIDMapT::const_iterator iter = m_row_map.find(row);
+  auto iter = m_row_map.find(row);
   if (iter != m_row_map.end()) {
     uuid_array_t uuid;
     iter->second.GetARep(uuid);
-    ItemListIter itemiter = m_core.Find(uuid);
+    auto itemiter = m_core.Find(uuid);
     if (itemiter == m_core.GetEntryEndIter())
       return NULL;
     return &itemiter->second;
@@ -405,7 +399,6 @@ CItemData *PWSGrid::GetItem(int row) const
   return NULL;
 }
 
-
 /*!
  * wxEVT_GRID_CELL_LEFT_DCLICK event handler for ID_LISTBOX
  */
@@ -449,10 +442,9 @@ void PWSGrid::OnChar( wxKeyEvent& evt )
   evt.Skip();
 }
 
-
-void PWSGrid::SaveSettings(void) const
+void PWSGrid::SaveSettings() const
 {
-  PWSGridTable* table = dynamic_cast<PWSGridTable*>(GetTable());
+  auto *table = dynamic_cast<PWSGridTable*>(GetTable());
   if (table)  //may not have been created/assigned
     table->SaveSettings();
 }
@@ -473,7 +465,6 @@ void PWSGrid::Clear()
   DeleteAllItems();
 }
 
-
 /*!
  * wxEVT_GRID_SELECT_CELL event handler for ID_LISTBOX
  */
