@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2018 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -147,7 +147,6 @@ PWSTreeCtrl::PWSTreeCtrl(wxWindow* parent, PWScore &core,
   Create(parent, id, pos, size, style);
 }
 
-
 /*!
  * PWSTreeCtrl creator
  */
@@ -161,7 +160,6 @@ bool PWSTreeCtrl::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos, co
   return true;
 }
 
-
 /*!
  * PWSTreeCtrl destructor
  */
@@ -172,7 +170,6 @@ PWSTreeCtrl::~PWSTreeCtrl()
 ////@end PWSTreeCtrl destruction
 }
 
-
 /*!
  * Member initialisation
  */
@@ -183,7 +180,6 @@ void PWSTreeCtrl::Init()
 ////@end PWSTreeCtrl member initialisation
 }
 
-
 /*!
  * Control creation for PWSTreeCtrl
  */
@@ -208,7 +204,7 @@ void PWSTreeCtrl::CreateControls()
   };
   const int Nimages = sizeof(xpmList)/sizeof(xpmList[0]);
 
-  wxImageList *iList = new wxImageList(13, 13, true, Nimages);
+  auto *iList = new wxImageList(13, 13, true, Nimages);
   for (int i = 0; i < Nimages; i++)
     iList->Add(wxBitmap(xpmList[i]));
   AssignImageList(iList);
@@ -294,23 +290,33 @@ wxTreeItemId PWSTreeCtrl::AddGroup(const StringX &group)
   return ti;
 }
 
-
 wxString PWSTreeCtrl::ItemDisplayString(const CItemData &item) const
 {
   PWSprefs *prefs = PWSprefs::GetInstance();
   const wxString title = item.GetTitle().c_str();
+
+  // Title is a mandatory field - no need to worry if empty
   wxString disp = title;
 
   if (prefs->GetPref(PWSprefs::ShowUsernameInTree)) {
     const wxString user = item.GetUser().c_str();
-    if (!user.empty())
-      disp += wxT(" [") + user + wxT("]");
+    // User is NOT a mandatory field - but show not present by empty brackets i.e. []
+    // if user wants it displayed
+    disp += wxT(" [") + user + wxT("]");
   }
 
   if (prefs->GetPref(PWSprefs::ShowPasswordInTree)) {
     const wxString passwd = item.GetPassword().c_str();
-    if (!passwd.empty())
-      disp += wxT(" {") + passwd + wxT("}");
+    // Password is a mandatory field - no need to worry if empty
+    disp += wxT(" {") + passwd + wxT("}");
+  }
+
+  if (item.IsProtected()) { 
+    disp += wxT(" #");
+#ifdef NOTYET
+    wxUniChar padlock(0x1f512);
+    disp +=padlock;
+#endif
   }
 
   return disp;
@@ -412,16 +418,15 @@ CItemData *PWSTreeCtrl::GetItem(const wxTreeItemId &id) const
   if (!id.IsOk())
     return NULL;
 
-  PWTreeItemData *itemData = dynamic_cast<PWTreeItemData *>(GetItemData(id));
+  auto *itemData = dynamic_cast<PWTreeItemData *>(GetItemData(id));
   // return if a group is selected
   if (itemData == NULL)
     return NULL;
 
-  ItemListIter itemiter = m_core.Find(itemData->GetUUID());
+  auto itemiter = m_core.Find(itemData->GetUUID());
   if (itemiter == m_core.GetEntryEndIter())
     return NULL;
   return &itemiter->second;
-
 }
 
 //overridden from base for case-insensitive sort
@@ -461,7 +466,7 @@ void PWSTreeCtrl::SortChildrenRecursively(const wxTreeItemId& item)
 wxTreeItemId PWSTreeCtrl::Find(const CUUID &uuid) const
 {
   wxTreeItemId fail;
-  UUIDTIMapT::const_iterator iter = m_item_map.find(uuid);
+  auto iter = m_item_map.find(uuid);
   if (iter != m_item_map.end())
     return iter->second;
   else
@@ -513,7 +518,6 @@ bool PWSTreeCtrl::Remove(const CUUID &uuid)
   }
 }
 
-
 void PWSTreeCtrl::SetItemImage(const wxTreeItemId &node,
                                const CItemData &item)
 {
@@ -557,7 +561,6 @@ void PWSTreeCtrl::OnTreectrlItemActivated( wxTreeEvent& evt )
   }
 }
 
-
 /*!
  * wxEVT_TREE_ITEM_MENU event handler for ID_TREECTRL
  */
@@ -588,7 +591,6 @@ void PWSTreeCtrl::OnGetToolTip( wxTreeEvent& evt )
   }
 }
 
-
 /*!
  * wxEVT_CHAR event handler for ID_TREECTRL
  */
@@ -605,7 +607,7 @@ void PWSTreeCtrl::OnChar( wxKeyEvent& evt )
 void PWSTreeCtrl::OnDBGUIPrefsChange(wxEvent& evt)
 {
   UNREFERENCED_PARAMETER(evt);
-  PasswordSafeFrame *pwsframe = dynamic_cast<PasswordSafeFrame *>(GetParent());
+  auto *pwsframe = dynamic_cast<PasswordSafeFrame *>(GetParent());
   wxASSERT(pwsframe != NULL);
   if (pwsframe->IsTreeView())
     pwsframe->RefreshViews();
@@ -677,7 +679,7 @@ void PWSTreeCtrl::OnEndLabelEdit( wxTreeEvent& evt )
     {
       wxTreeItemId groupItem = evt.GetItem();
       if (groupItem.IsOk()) {
-        PWTreeItemData* data = dynamic_cast<PWTreeItemData *>(GetItemData(groupItem));
+        auto *data = dynamic_cast<PWTreeItemData *>(GetItemData(groupItem));
         if (data && data->BeingAdded()) {
           // A new group being added
           FinishAddingGroup(evt, groupItem);
@@ -769,7 +771,6 @@ void PWSTreeCtrl::FinishRenamingGroup(wxTreeEvent& evt, wxTreeItemId groupItem,
     wxTreeCtrl::SelectItem(newItem);
 }
 
-
 /*!
  * wxEVT_COMMAND_TREE_SEL_CHANGED event handler for ID_TREECTRL
  */
@@ -802,4 +803,3 @@ void PWSTreeCtrl::SetFilterState(bool state)
   if (root)
     ColourChildren(this, root, *colour);
 }
-
